# -*- coding: utf-8 -*-
import os
import pySMS.html.pathdata.prodChooser as stprodchooser
import pySMS.html.pathdata.stockTabTable as sttable
from pySMS.utils.excepetions import ProductChooserAddError, DataRetreivalError, InvalidItemError
from pySMS.data.export import Export

class Webpage:

    def __init__(self):
        self.path = os.path.dirname(os.path.realpath(__file__))

    class Logic:

        def __init__(self):
            self.exp = Export()
            self.item_library = self.exp.convert_to_dict(
                self.exp.unzip_from_gzip(
                    self.exp.data_files_list['ci_data_library.json.gz'])['content'].decode())

        @staticmethod
        def product_chooser_add_item(item_dict, table_dict):
            product = item_dict['prodNumber']
            description = item_dict['description']
            row_count = len(table_dict)
            match_count = 0
            match_dict = {}

            for k, v in table_dict.items():
                for _item in v:
                    if product == _item and description in v:
                        match_count += 1
                        match_dict[k] = {'product': product, 'description': description}
                    elif product == _item and 'None' in v:
                        match_count += 1
                        match_dict[k] = {'product': product, 'description': description}

            if match_count != 0:
                if match_count < 1:
                    return {'action': 'none', 'reason': 'no match found'}
                if match_count == 1:
                    for k, v in match_dict.items():
                        if int(str(k.strip('row'))) == 1:
                            return {'action': 'click', 'selector_count': 1, 'selectors': [stprodchooser.prodchooserAdd]}
                        elif int(str(k.strip('row'))) % 2 == 0:
                            if int(str(k.strip('row'))) == row_count:
                                return {'action': 'click', 'selector_count': 2, 'selectors': [
                                    stprodchooser.prodChooserAddLastFront + str(k.strip('row')) + stprodchooser.prodChooserAddLastBack,
                                    stprodchooser.prodChooserAddFront + str(k.strip('row')) + stprodchooser.prodChooserAddEvenBack]}
                            else:
                                return {'action': 'click', 'selector_count': 1, 'selectors': [
                                    stprodchooser.prodChooserAddFront + str(k.strip('row')) + stprodchooser.prodChooserAddEvenBack]}
                        elif int(str(k.strip('row'))) % 2 != 0:
                            return {'action': 'click', 'selector_count': 1, 'selectors': [
                                stprodchooser.prodChooserAddFront + str(k.strip('row')) + stprodchooser.prodChooserAddOddBack]}
            else:
                raise ProductChooserAddError(' product_chooser_add_item(item_dict, table_dict): No Match Found')

        @staticmethod
        def stock_tab_table_checkboxes(n):
            if int(n) == 1:
                return sttable.stTableCheckBoxInit
            elif int(n) % 2 == 0:  # is an even row
                return sttable.stTableCheckBoxRowEvenFront + n + sttable.stTableCheckBoxRowEvenBack
            else:  # is an odd row
                return sttable.stTableCheckBoxRowOddFront + n + sttable.stTableCheckBoxRowOddBack

        def get_item_info(self, item):
            for key, value in self.item_library.items():
                for book_key, book_value in value.items():
                    for product, page in book_value.items():
                        if product == item:
                            return page
                        else:
                            pass


        def bool_data_from_table(self, t_dict, data_list):
            item_id_dict = {}

            def format_bin_id(bin):
                if bin != "location_bin":
                    return bin
                else:
                    return ' '.join([bin.split("_")[0].title(), bin.split("_")[1].title()])


            item_info = self.get_item_info(t_dict['prodnumber'])


            if data_list[3] == "none":
                d_dict = {'case_number': str(data_list[0]).rstrip('.0'),
                          'lot_number': str(data_list[1]).rstrip('.0'),
                          'serial_number': str(data_list[2]).rstrip('.0')}

                if item_info == None:
                    return {'match_found': False, 'data': None}

                if item_info['data_parent'].split('_')[0] in ['kit', 'tray']:
                    if d_dict['serial_number'] != '' and d_dict['lot_number'] == '':
                        item_id_dict = {'type': 'serial', 'id': "Serial " + str(d_dict['serial_number']) + " ~ "}

                elif item_info['data_parent'].split('_')[0] in ['component', 'piece']:
                    if d_dict['serial_number'] == '' and d_dict['lot_number'] != '':
                        item_id_dict = {'type': 'lot', 'id': str(d_dict['lot_number'])}

                if item_id_dict['type'] == 'serial':
                    if item_id_dict['id'] in t_dict['lot_serial']:
                        return {'match_found': True, 'data': {'info': item_id_dict, 'row': t_dict['rowNumber'],
                                                              'checkbox_selector': self.stock_tab_table_checkboxes(
                                                                  t_dict['rowNumber']),
                                                              'case': t_dict['container']}}
                    if item_id_dict['id'] not in t_dict['lot_serial']:
                        return {'match_found': False, 'data': None}

                elif item_id_dict['type'] == 'lot':
                    if d_dict['case_number'] in t_dict['container']:
                        return {'match_found': True, 'data': {'info': item_id_dict, 'row': t_dict['rowNumber'],
                                                              'checkbox_selector': self.stock_tab_table_checkboxes(
                                                                  t_dict['rowNumber']),
                                                              'case': t_dict['container']}}
                    elif d_dict['case_number'] not in t_dict['container']:
                        return {'match_found': False, 'data': None}


            else:
                d_dict = {'case_number': str(data_list[0]).rstrip('.0'),
                          'lot_number': str(data_list[1]).rstrip('.0'),
                          'serial_number': str(data_list[2]).rstrip('.0'),
                          'bin': str(data_list[3])}

                if item_info['data_parent'].split('_')[0] in ['kit', 'tray']:
                    if d_dict['serial_number'] != '' and d_dict['lot_number'] == '':
                        item_id_dict = {'type': 'serial', 'id': "Serial " + str(d_dict['serial_number']) + " ~ ", 'bin': format_bin_id(d_dict['bin'])}

                elif item_info['data_parent'].split('_')[0] in ['component', 'piece']:
                    if d_dict['serial_number'] == '' and d_dict['lot_number'] != '':
                        item_id_dict = {'type': 'lot', 'id': str(d_dict['lot_number']), 'bin': format_bin_id(d_dict['bin'])}

                if item_id_dict['type'] == 'serial':
                    if item_id_dict['id'] in t_dict['lot_serial']:
                        if item_id_dict['bin'] in t_dict['container']:
                            return {'match_found': True, 'data': {'info': item_id_dict, 'row': t_dict['rowNumber'],
                                                                  'checkbox_selector': self.stock_tab_table_checkboxes(t_dict['rowNumber']),
                                                                  'case': t_dict['container']}}
                    if item_id_dict['id'] not in t_dict['lot_serial']:
                        return {'match_found': False, 'data': None}

                elif item_id_dict['type'] == 'lot':
                    if d_dict['lot_number'] in t_dict['lot_serial']:
                        if item_id_dict['bin'] in t_dict['container']:
                            return {'match_found': True, 'data': {'info': item_id_dict, 'row': t_dict['rowNumber'],
                                                                  'checkbox_selector': self.stock_tab_table_checkboxes(t_dict['rowNumber']),
                                                                  'case': t_dict['container']}}
                    elif d_dict['case_number'] not in t_dict['container']:
                        return {'match_found': False, 'data': None}



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 28 08:11:44 2016

@author: doarni
"""

import os
import time
import json
import pySMS.pgsql.cipher as cipher
import psycopg2
import traceback

from colorama import init, Fore

init(autoreset=True)


class Postgres:
    
    def __init___(self):
        self.path = os.path.dirname(os.path.realpath(__file__))

        self.con = None
        self.cur = None
        self._user = None
        self._pass = None
        self._host = None
        
        self.postgres_data = None
        
    def load_postgres_data(self):
        with open(os.path.dirname(os.path.realpath(__file__)) + '\\postgres_data.json', 'r') as fp:
            self.postgres_data = json.load(fp)
        self._user = cipher.decipher(self.postgres_data['data']['value1'].encode())
        self._pass = cipher.decipher(self.postgres_data['data']['value2'].encode())
        self._host = cipher.decipher(self.postgres_data['data']['value3'].encode())

    def test_connection(self, N=5, wait=5, connection="DEFAULT",username="", password="", host="", database="postgres", verbose=True):
        if connection.upper() == "DEFAULT":
            print(Fore.LIGHTYELLOW_EX + 'Testing connection to postgres') if verbose == True else None
            for i in range(0,N):
                try:
                    self.con = psycopg2.connect("dbname=" + database + " user=" + self._user + " host=" + self._host + " password=" + self._pass)
                    self.cur = self.con.cursor()
                    print(Fore.LIGHTMAGENTA_EX + 'Connection Available!\n') if verbose == True else None
                    self.con.close()
                    break
                except Exception as e:
                    log = format(str(e))
                    log_ = traceback.format_exc()+'.\n'+log
                    print(Fore.LIGHTRED_EX + 'Could not establish connection after [' + str(i + 1) + '] tries.') if verbose == True else None
                    print(Fore.LIGHTRED_EX + 'Full Trackback: ' + Fore.LIGHTWHITE_EX + log_) if verbose == True else None
                    time.sleep(wait)
        else:
            print(Fore.LIGHTYELLOW_EX + 'Testing connection.') if verbose == True else None
            for i in range(0,N):
                try:
                    self.con = psycopg2.connect("dbname=" + database + " user=" + username + " host=" + host + " password=" + password)
                    self.cur = self.con.cursor()
                    print(Fore.LIGHTMAGENTA_EX + 'Connection Available!\n') if verbose == True else None
                    self.con.close()
                    break
                except Exception as e:
                    log = format(str(e))
                    log_ = traceback.format_exc()+'.\n'+log
                    print(Fore.LIGHTRED_EX + 'Could not establish connection after [' + str(i + 1) + '] tries.') if verbose == True else None
                    print(Fore.LIGHTRED_EX + 'Full Trackback: ' + Fore.LIGHTWHITE_EX + log_) if verbose == True else None


    def establish_connection(self):
        self.con = psycopg2.connect("dbname=postgres user=" + self._user + " host=" + self._host + " password=" + self._pass)
        self.cur = self.con.cursor()
        print(Fore.LIGHTGREEN_EX + 'Successfully Connected!\n')
        
    def close_connection(self):
        print(Fore.LIGHTYELLOW_EX + 'Closing connection to postgres')
        try:
            self.con.close()
            print(Fore.LIGHTGREEN_EX + 'Connection Closed.\n')            
        except:
            print(Fore.LIGHTRED_EX + 'Unable to close connection to host: ' + self._host)

    def execute(self, query):
        if self.con == None:
            print(Fore.LIGHTRED_EX + 'Unable to execute query. No connection has been established with the server.\n')
        else:
            self.cur.execute(query)
            self.con.commit()
            try:
                data = self.cur.fetchall()
                print(Fore.LIGHTMAGENTA_EX + 'Query Completed')
                return data
            except:
                print(Fore.LIGHTMAGENTA_EX + 'No data recieved')



# -*- coding: utf-8 -*-
"""
Created on Thu Nov 17 00:06:01 2016

@author: doarni
"""
from prettytable import PrettyTable
from colorama import init, Fore
init(autoreset=True)

class tables():
    
    def intro():
        x = PrettyTable()
        
        def colorTxt(txt):
            return Fore.LIGHTCYAN_EX + txt
        
        x.field_names = [colorTxt("Credentials")]
        x.add_row([colorTxt("Please Login with your SMS credentials.")])
        x.align = "l"
        x.junction_char = '#'
        
        print(x)
        
    def head():
        x = PrettyTable()
       
        def colorTxt(txt):
            return Fore.LIGHTGREEN_EX + txt
        x.field_names = [colorTxt("PYSMS MACROS FOR ZBSMS")]
        x.add_row([colorTxt("pySMS is a keyboard macro program")])
        x.add_row([colorTxt("to make navigating ZBSMS faster ")])
        x.add_row([colorTxt("and more intuitive. Press F10")])
        x.add_row([colorTxt("for a list of commands!")])
        x.add_row([colorTxt("Created by: Ian Doarn")])
        x.add_row([colorTxt("EMAIL: Ian.Doarn@zimmerbiomet.com")])
        x.align = "l"
        x.junction_char = '%'
        x.padding_width = 0
        print(x)
            
    def stockTableDisplay(p, d, ls, ct, lc, lt):
        x = PrettyTable()
        
        def whiteTxt(txt):
            return Fore.LIGHTWHITE_EX + txt

        def greenTxt(txt):
            return Fore.LIGHTGREEN_EX + txt
        
        x.field_names = [greenTxt("Product"), 
                         greenTxt("Desc"), 
                         greenTxt("Lot // Serial Number"), 
                         greenTxt("Container"), 
                         greenTxt("Location"),
                         greenTxt("Location Type")]
                         
        x.add_row([whiteTxt(p), 
                   whiteTxt(d), 
                   whiteTxt(ls), 
                   whiteTxt(ct), 
                   whiteTxt(lc), 
                   whiteTxt(lt)])
                   
        x.align = "l"
        x.junction_char = '*'
        x.padding_width = 0
        print(x)
    
    def formatDictotList(dict):
        list = []
        for key, val in dict.items():
            list.append((str(key) + ': ' , str(val)))
        return list
        
    def formatListToTable(dict, listX, color):
        string = ""
        list = []
        for a in range(len(listX)):
            list.append(listX[a][0] + color + listX[a][1])
        for i in range(len(list[0]) + 5):
            string = string + "-"
        print(string)
        for item in range(len(list)):
            print('> ' + list[item], sep=' | ')
        print(string)




# -*- coding: utf-8 -*-
"""
Created on Tue Nov 15 15:47:29 2016

@author: doarni
"""

#Special Keyboard KeyID's
#------------------

#ESC
#------
esc = 27

#CTRL 
#------
lctrl = 162
rctrl = 163

#SHIFT
#------
lshift = 160
rshift = 161

#ALT
#------
lalt = 164
ralt = 165

#ENTER
#------
enter = 13

#TAB
#------
tab = 9

#tilde
#------
tilde = 192

#ARROW KEYS
#------
arrowleft = 37
arrowright = 39
arrowup = 38
arrowdown = 40

#HOME
#------
home = 36

#F Keys
#------
f1 = 112
f2 = 113
f3 = 114
f4 = 115
f5 = 116
f6 = 117
f7 = 118
f8= 119
f9 = 120
f10 = 121
f11 = 122
f12 = 123

#NUMPAD
#------
NUM_0 = 96
NUM_1 = 97
NUM_2 = 98
NUM_3 = 99
NUM_4 = 100
NUM_5 = 101
NUM_6 = 102
NUM_7 = 103
NUM_8 = 104
NUM_9 = 105
NUM_add = 107
NUM_subtract = 109
NUM_multiply = 106
NUM_divide = 111
NUM_decimalPoint = 110

#VK_KEYS
#------
VK_F1 = 'VK_F1'
VK_F2 = 'VK_F2'
VK_F3 = 'VK_F3'
VK_F4 = 'VK_F4'
VK_F5 = 'VK_F5'
VK_F6 = 'VK_F6'
VK_F7 = 'VK_F7'
VK_F8 = 'VK_F8'
VK_F9 = 'VK_F9'
VK_F10 = 'VK_F10'
VK_F11 = 'VK_F11'
VK_F12 = 'VK_F12'
VK_LCONTROL = 'VK_LCONTROL'
VK_LALT = 'VK_LMENU'
VK_LSHIFT = 'VK_LSHIFT'
VK_PGDOWN = 'VK_NEXT'
VK_PRINT = 'VK_PRINT'
VK_PGUP = 'VK_PRIOR'
VK_RCONTROL = 'VK_RCONTROL'
VK_RALT = 'VK_RMENU'
VK_RSHIFT = 'VK_RSHIFT'
VK_PRINTSCREEN = 'VK_SNAPSHOT'
VK_TAB = 'VK_TAB'



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 28 13:16:34 2016

@author: doarni
"""

import lxml.html as Lhtml

class parse():
        
    def returnClassName(htmlData, path, indexNumber): #return class at given list index
        html = Lhtml.fromstring(htmlData)
        list = html.cssselect(path)
        return list[indexNumber].get('class')
        
    def returnClassData(htmlData, path): #return classes as a dictionary
        html = Lhtml.fromstring(htmlData)
        list = html.cssselect(path)
        classes = []
        for i in len(list):
            classes.append(list[i].get('class'))
        return {'path': path, 'data': classes}

    def returnInfo(htmlData, path, indexNumber, selectorType): #return list of data at end of path
        html = Lhtml.fromstring(htmlData)
        list = html.cssselect(path)
        return list[indexNumber].get(selectorType)
        
    def returnTextInfo(htmlData, path):
        html = Lhtml.fromstring(htmlData)
        list = html.cssselect(path)
        for item in list:           
            return [item.text_content(),item.get('value')]
    


upSaveButton = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > div > table > tbody > tr:nth-child(8) > td > table > tbody > tr > td:nth-child(2) > span > div > div.GLNOMTPDN4C.GLNOMTPDK4C'
upUser = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td.GLNOMTPDLX > table > tbody > tr > td.GLNOMTPDIX > span'
upUserSettingsButton = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > div > div.GLNOMTPDBGB > table > tbody > tr:nth-child(1) > td:nth-child(2) > table > tbody > tr > td.GLNOMTPDNFB > span'



# -*- coding: utf-8 -*-
"""
Created on Wed Nov 16 17:08:22 2016

@author: doarni
"""
from pySMS.ui.tk import widget

class prompts():
    
    def loginInfo():
        w = widget()
        username = input("Username: ")
        password = w.get('Password', "*")
        return username, password



username = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td > div.GLNOMTPDDLI > table > tbody > tr:nth-child(2) > td.GLNOMTPDDS > input'
url = 'http://www.zimmersms.com'
loginbutton = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td > div.GLNOMTPDDLI > table > tbody > tr:nth-child(5) > td > table > tbody > tr > td > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
password = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td > div.GLNOMTPDDLI > table > tbody > tr:nth-child(3) > td.GLNOMTPDDS > input'



# -*- coding: utf-8 -*-
"""
Created on Wed Dec  7 12:02:33 2016

@author: doarni
"""

from tkinter import *
from tkinter.filedialog import askopenfilename   

fname = "unassigned"

def dialog():    
    def openFile():
        global fname
        fname = askopenfilename(filetypes = (("Excel files", "*.xlsx;*.xls"),
                                             ("All files", "*.*")))
        root.destroy()

    root = Tk()
    Button(root, text='Load file', command = openFile).pack(fill=X)
    mainloop()
    return fname
    
    


    


# -*- coding: utf-8 -*-

import ctypes
import os
import json

import traceback
import pySMS.messagebox as mbox


class Handles(object):

    def __init__(self):
        self.path = os.path.dirname(os.path.realpath(__file__))

    class Data:

        def __init__(self):
            super(Handles.Data, self).__init__()
            self.path = os.path.dirname(os.path.realpath(__file__))

        def load_json_data(self):
            with open(self.path + '\\html\\pathdata\\json\\siteSettings.json', 'r') as fp:
                return json.load(fp)

    class Errors:

        def __init__(self):
            super(Handles.Errors, self).__init__()

        def err_basic(self, msg, title):
            mbox.msgBox(title, msg, 'MB_OK', 'ICON_STOP')

        def err_traceback(self, e, message):
            mbox.msgBox('Uh Oh!', message + ' Traceback: ' + str(e), 'MB_OK', 'ICON_EXCLAIM')

        def err_full_traceback(self, e, message):
            log = format(str(e))
            log_ = traceback.format_exc()+'.\n'+log
            mbox.msgBox('Uh Oh!', message + ' \n Traceback: ' + str(log_), 'MB_OK', 'ICON_EXCLAIM')

        def err_explicit(self, e, message, title, button, icon):
            mbox.msgBox(title, message + 'Traceback: ' + str(e), button, icon)

        def err_custom(self, message, title, button, icon):
            mbox.msgBox(title, message, button, icon)

    class Helpers:

        def __init__(self):
            super(Handles.Helpers, self).__init__()

        def return_module_path(self):
            import pySMS
            path = pySMS.__file__.split('\\')
            return '\\'.join(path[:len(path) - 1])




"""
Used to reference dll files

DO NOT DELETE THIS FILE
"""
import os

DLL_PATH = os.path.dirname(os.path.realpath(__file__))
PYTHON_EMBEDDINGTEST_DLL = "Python.EmbeddingTest.dll"
PYTHON_RUNTIME_DLL = "Python.Runtime.dll"
PYTHON_TEST_DLL = "Python.Test.dll"
SMS_DLL = "smsdll"





# -*- coding: utf-8 -*-
"""
Created on Wed Dec 28 07:33:37 2016

@author: doarni
"""

import os
from pySMS.pgsql.postgres import Postgres as Postgres
from colorama import init, Fore
init(autoreset=True)

class data_builder():
    
    def __init__(self):

        self.path = os.path.dirname(os.path.realpath(__file__))
        self.ci_kit_data_file = self.path + "\\sql\\ci_kit_data.sql"
        self.ci_piece_data_file = self.path + "\\sql\\ci_piece_data.sql"
        self.ci_component_data_file = self.path + "\\sql\\ci_component_data.sql"
        self.ci_tray_data_file = self.path + "\\sql\\ci_tray_data.sql"

        self.ci_kit_data = None
        self.ci_component_data = None
        self.ci_piece_data = None
        self.ci_tray_data = None
        
        self.pgsql = None
        
    def load_pgsql(self):
        self.pgsql = Postgres()
        self.pgsql.load_postgres_data()
        self.pgsql.test_connection()
        self.pgsql.establish_connection()
        
    def close_pgsql(self):
        if self.pgsql == None:
            print(Fore.LIGHTRED_EX + 'Postgres is not connected')
        else:
            self.pgsql.close_connection()
        
    def load_ci_query_text(self):
        kit_query_txt = ""
        tray_query_txt = ""
        component_query_txt = ""
        piece_query_txt = ""
        
        #load kit query text
        with open(self.ci_kit_data_file, 'r')as f:
            kit_query_txt = f.read().replace('\n', ' ')
        kit_query_txt = kit_query_txt.replace('\t', ' ')
        kit_query_txt = kit_query_txt[3:]
        
        #load tray query text
        with open(self.ci_tray_data_file, 'r')as f:
            tray_query_txt = f.read().replace('\n', ' ')
        tray_query_txt = tray_query_txt.replace('\t', ' ')
        tray_query_txt = tray_query_txt[3:]        
        
        #load component query text
        with open(self.ci_component_data_file, 'r')as f:
            component_query_txt = f.read().replace('\n', ' ')
        component_query_txt = component_query_txt.replace('\t', ' ')
        component_query_txt = component_query_txt[3:]

        #load piece query text
        with open(self.ci_piece_data_file, 'r')as f:
            piece_query_txt = f.read().replace('\n', ' ')
        piece_query_txt = piece_query_txt.replace('\t', ' ')
        piece_query_txt = piece_query_txt[3:]
               
        return {"kit": kit_query_txt,
                "tray": tray_query_txt,
                "component": component_query_txt,
                "piece": piece_query_txt}

    def load_ci_data(self):
        dict = self.load_ci_query_text()
        
        print(Fore.LIGHTYELLOW_EX + 'Loading data.')

        print(Fore.LIGHTYELLOW_EX + 'Loading kit data')
        self.ci_kit_data = self.pgsql.execute(dict['kit'])
        print(Fore.LIGHTGREEN_EX + 'Data loaded.\n')

        print(Fore.LIGHTYELLOW_EX + 'Loading component data.')
        self.ci_component_data = self.pgsql.execute(dict['component'])
        print(Fore.LIGHTGREEN_EX + 'Data loaded.\n')

        print(Fore.LIGHTYELLOW_EX + 'Loading piece data.')        
        self.ci_piece_data = self.pgsql.execute(dict['piece'])
        print(Fore.LIGHTGREEN_EX + 'Data loaded.\n')

        print(Fore.LIGHTYELLOW_EX + 'Loading tray data.')       
        self.ci_tray_data = self.pgsql.execute(dict['tray'])       
        print(Fore.LIGHTGREEN_EX + 'Data loaded.\n')
        
    def return_ci_data(self):
        return {"kit_data": self.ci_kit_data,
                "piece_data": self.ci_piece_data,
                "tray_data": self.ci_tray_data,
                "component_data": self.ci_component_data}



# -*- coding: utf-8 -*-
import pySMS.configparse
import pySMS.reference
import pySMS.keys
import pySMS.livery
import pySMS.messagebox
import pySMS.prompts
import pySMS.selenium
import pySMS.utils
import pySMS.json
import pySMS.data
import pySMS.data.export
import pySMS.handlers
import pySMS.utils.logger
import pySMS.utils.excepetions
import pySMS.utils.networkdrive
import pySMS.utils.logic
import pySMS.utils.logic.webpage
import pySMS.excel
import pySMS.excel.loadFile
import pySMS.excel.parseFile
import pySMS.excel.ui.openfile
import pySMS.html
import pySMS.html.pathdata.adjustStock
import pySMS.html.pathdata.loginPage
import pySMS.html.pathdata.mainPage
import pySMS.html.pathdata.prodChooser
import pySMS.html.pathdata.siteSettings
import pySMS.html.pathdata.stockTab
import pySMS.html.pathdata.stockTabMutate
import pySMS.html.pathdata.stockTabTable
import pySMS.html.pathdata.userSettings
import pySMS.html.pathdata.json
import pySMS.ui
import pySMS.ui.openfile
import pySMS.ui.tk
import pySMS.req
import pySMS.req.prereq
import pySMS.req.verify
import pySMS.req.ppb
import pySMS.pgsql
import pySMS.pgsql.build_library
import pySMS.pgsql.cipher
import pySMS.pgsql.load_data
import pySMS.pgsql.postgres
import pySMS.pgsql.sql


prodChooserAddEvenBack = ') > td.GLNOMTPDHM.GLNOMTPDEO > div > table > tbody > tr > td > table > tbody > tr > td.GLNOMTPDBYI > span'
prodchooserSearchButton = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr:nth-child(1) > td.GLNOMTPDAS  > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
prodChooserAddOddBack = ') > td.GLNOMTPDGM.GLNOMTPDEO > div > table > tbody > tr > td > table > tbody > tr > td.GLNOMTPDBYI > span'
prodchooserSearchBar = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr:nth-child(1) > td.GLNOMTPDP-D > input'
prodChooserTableData = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody'
prodChooserDescBack = ') > td:nth-child(3) > div'
prodChooserAddLastBack = ') > td.GLNOMTPDGM.GLNOMTPDEO.GLNOMTPDMM > div > table > tbody > tr > td > table > tbody > tr > td.GLNOMTPDBYI > span'
prodChooserTRpathBack = ')'
prodChooserAddFront = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
prodchooserAdd = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr > td.GLNOMTPDHM.GLNOMTPDEO.GLNOMTPDBP > div > table > tbody > tr > td > table > tbody > tr > td.GLNOMTPDBYI > span'
prodChooserAddLastFront = 'body > div.GLNOMTPDL-D> div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child( '
prodChooserTRpathFront = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
prodChooserResults = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td.GLNOMTPDBAE'
prodChooserDescFront = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
prodchooserFinishButton = 'body > div.GLNOMTPDL-D > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > table > tbody > tr > td.GLNOMTPDD0D > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
prodchooserSelect = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div:nth-child(1) > div:nth-child(2) > div.GLNOMTPDCW > table > tbody > tr > td > table > tbody > tr > td:nth-child(2) > span'



# -*- coding: utf-8 -*-

import logging
import os
import sys
import datetime


LEVELS = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL}


class Logger(object):

    def __init__(self, name):
        self.path = os.path.dirname(os.path.realpath(__file__))
        self.date = str(datetime.datetime.now().strftime("%Y-%m-%d"))
        self.name = name

        self.setup('logs', is_dir=True)
        self.setup('logs\\' + self.name + '_' + self.date + '_log.log', is_file=True)

        self.logger = logging.getLogger(self.name)
        self.logger.setLevel(logging.INFO)
        self.log_handle = logging.FileHandler('logs\\' + self.name + '_' + self.date + '_log.log')
        self.log_formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.log_handle.setFormatter(self.log_formatter)
        self.logger.addHandler(self.log_handle)

        self.current_running_log = {}


    @staticmethod
    def get_current_data_stamp():
        return str(datetime.datetime.now().strftime("%Y-%m-%d"))

    @staticmethod
    def setup(path, is_file=False, is_dir=False):
        if is_dir == True:
            if os.path.exists(path):
                pass
            else:
                os.mkdir(path)
        elif is_file == True:
            if os.path.isfile(path):
                os.system('del /s /q ' + path)
                with open(path, 'w')as f:
                    f.close()
            else:
                with open(path, 'w')as f:
                    f.close()

    def set_log_level(self, level):
        self.log_handle.setLevel(level)

    def write_log_file(self, _log):
        log_dict = _log

        for key, value in log_dict.items():
            try:
                if key == 'error':
                    for item in value:
                        self.set_log_level(LEVELS['error'])
                        self.logger.error(str(item))

                if key == 'debug':
                    for item in value:
                        self.set_log_level(LEVELS['debug'])
                        self.logger.debug(str(item))

                if key == 'warning':
                    for item in value:
                        self.set_log_level(LEVELS['warning'])
                        self.logger.warning(str(item))

                if key == 'info':
                    for item in value:
                        self.set_log_level(LEVELS['info'])
                        self.logger.info(str(item))

                if key == 'critical':
                    for item in value:
                        self.set_log_level(LEVELS['critical'])
                        self.logger.critical(str(item))

            except Exception as e:
                self.set_log_level(LEVELS['debug'])
                self.logger.debug(' :: Exception has occurred => ' + str(e))






# -*- coding: utf-8 -*-
import os
import sys
import zipfile
from colorama import init, Fore
init(autoreset=True)

class installer():
    def __init__(self):
        self.path = os.path.dirname(os.path.realpath(__file__))
    
    def readZipFile(self, file):   
        with zipfile.ZipFile(file, "r") as f:
            i = 1
            for name in f.namelist():
                print(str(i) + ') ' + name)
                i += 1
            print(Fore.LIGHTYELLOW_EX + 'Files in ' + file + ': ' + Fore.LIGHTCYAN_EX + str(i - 1))
                
    def unzip(self, file, target):
        with zipfile.ZipFile(file,"r") as zip_ref:
            zip_ref.extractall(target)
            
            
    def compileFile(self, file, location):
        print(Fore.LIGHTYELLOW_EX + 'Compiling: ' + Fore.LIGHTCYAN_EX + file)
        os.system("pyinstaller --specpath " + location + " --distpath " + location + "//dist --workpath " + location + "//build " + file)

    def pipInstallUpdateList(self, list, singleItem=False):
        for module in list:
            print(Fore.LIGHTYELLOW_EX + 'Installing: ' + Fore.LIGHTCYAN_EX + module)
            os.system("pip install " + module + " --no-cache-dir")
        for module in list:
            print(Fore.LIGHTYELLOW_EX + 'Updating: ' + Fore.LIGHTCYAN_EX + module)
            os.system("pip install " + module + " --upgrade --no-cache-dir")            
    
    def pipInstallUpdateItem(self, module):
        print(Fore.LIGHTYELLOW_EX + 'Installing: ' + Fore.LIGHTCYAN_EX + module)
        os.system("pip install " + module + " --no-cache-dir")
        print(Fore.LIGHTYELLOW_EX + 'Updating: ' + Fore.LIGHTCYAN_EX + module)
        os.system("pip install " + module + " --upgrade --no-cache-dir")            
     
    def pipInstallWheels(self, location, wheel):
        print(Fore.LIGHTYELLOW_EX + 'Installing: ' + Fore.LIGHTCYAN_EX + wheel)
        os.system("pip install " + location + " --no-cache-dir")     
        
    def writeBatLauncher(self, location, batFileName, pyFileName):
        with open(location + '\\' + batFileName,'w')as f:
            f.write('cd ' + location + ' & python ' + pyFileName)
        f.close()
        
       
if __name__ == '__main__':
    
    import argparse
    inst = installer()
    
    parser = argparse.ArgumentParser(description = '[PPB] Python Program Builder. A simple installer built with Python 3.5.6')
    
    parser.add_argument('-u','--unzip', nargs=2, help='Unzip files from zipfile to directory' , required=False, metavar=('[FILE]', '[DESTINATION]'))
    parser.add_argument('-c','--compile', nargs=2, help='Compile given file to exe using pyinstaller', required=False, type=str,metavar=('[FILE]' ,'[DESTINATION]'))
    parser.add_argument('-p','--parse', nargs=1, help='Parse files in zip file and print to console', required=False, metavar='[FILE]')
    parser.add_argument('-e','--update', nargs='+', help='Install and update a list of modules. use /f to supply a txt file', required=False, metavar='[LIST OF MODULES]')
    parser.add_argument('-w','--wheel', nargs=2, help='Install Wheel', required=False, metavar=('[PATH]', '[WHEEL]'))
    parser.add_argument('-b','--create-bat-file', nargs=3, help='Create batch file to launch python script', required=False, metavar=('[DESTINATION]','[BAT FILE NAME]','[PY FILE NAME]'))
    parser.add_argument('-os','--os-call-process', nargs='+', help='Run OS Command', required=False, metavar=('[COMMAND]', '[INPUT 1] [INPUT 2] [SWITCH1.SWITCH2.ECT]'))
    parser.add_argument('-md','--mkdir', nargs=1, help='Make Directory', required=False, metavar='[DIRECTORY PATH]')    
    parser.add_argument('-rd','--rmdir', nargs=1, help='Remove Directory', required=False, metavar='[DIRECTORY PATH]')    
    parser.add_argument('-del','--del', nargs=2, help='Delete item', required=False, metavar=('[SWITCH1.SWITCH2.SWITCH3.ect]', '[FILE]'))
    
    args = vars(parser.parse_args())
        
    if args['mkdir']:
        os.mkdir(args['mkdir'][0])
        
    if args['rmdir']:
        os.rmdir(args['rmdir'][0])        
    
    if args['del']:
        list = args['del'][0].split('.')
        switches = '/' + ' /'.join(list)
        os.system('del ' + switches + ' ' + args['del'][1])   
    
    if args['unzip']:
        if '.zip' not in args['unzip'][0]:
            print(Fore.LIGHTRED_EX + 'A zip file is required')
            sys.exit()
        else:
            if args['unzip'][1] == os.path.dirname(os.path.realpath(__file__)):
                print(Fore.LIGHTRED_EX + 'Can not unzip files to current directory')
            else:
                print('Unzipping ' + args['unzip'][0] + ' to ' + args['unzip'][1])
                inst.unzip(args['unzip'][0],args['unzip'][1])
    
    if args['compile']:
        inst.compileFile(args['compile'][0], args['compile'][1])
       
    if args['os_call_process']:
        if args['os_call_process'][0].lower() == 'xcopy':
            list = args['os_call_process'][3].split('.')
            switches = ' /'.join(list)
            os.system('xcopy ' + args['os_call_process'][1] + ' ' + args['os_call_process'][2] + ' /' + switches)
        else:
            print(Fore.LIGHTRED_EX + 'No Command Supplied')
            
    if args['parse']:
        print('Files in ' + args['parse'][0] + ':')
        inst.readZipFile(args['parse'][0])

    if args['update']:
        if args['update'][0].lower() == '/f':
            if '.txt' not in args['update'][1]:
                print(Fore.LIGHTRED_EX + 'File supplied not a .txt file')
            else:
                file = args['update'][1]
                list = [line.strip() for line in open(file)]
                for item in list:
                    inst.pipInstallUpdateItem(item)
        else:            
            inst.pipInstallUpdateList(args['update'])
        
    if args['wheel']:
        inst.pipInstallWheels(args['wheel'][0], args['wheel'][1])
        
    if args['create_bat_file']:
        inst.writeBatLauncher(args['create_bat_file'][0],
                              args['create_bat_file'][1],
                              args['create_bat_file'][2])


# -*- coding: utf-8 -*-
"""
Created on Wed Dec 28 08:40:38 2016

@author: doarni
"""

import binascii as basc
import os


def cipher(txt):
    hex = basc.hexlify(txt.encode())  
    return basc.b2a_hex(hex)

def decipher(data):
    hex = basc.a2b_hex(data)
    return str(basc.unhexlify(hex), 'ascii')
 
def build_cipher_file():
    import json
    
    dict = {}
    
    value1 = str(cipher(input(decipher(b'35353733363537323665363136643635') + ' ')), 'ascii')
    value2 = str(cipher(input(decipher(b'35303631373337333737366637323634') + ' ')), 'ascii')
    value3 = str(cipher(input(decipher(b'3438366637333734') + ' ')), 'ascii')

    dict['data'] = {'value1': value1, 'value2': value2, 'value3': value3}
    
    os.system('cls')    
    
    print('Writing file')
    with open(os.path.dirname(os.path.realpath(__file__)) + '\\postgres_data.json', 'w')as fp:
        json.dump(dict, fp, sort_keys=False, indent=5)
    print('Complete')

if __name__ == '__main__':
    
    import argparse
    
    parser = argparse.ArgumentParser(description = 'Simpler Hex Cipher')
    parser.add_argument('-e','--encrypt', nargs=1, help='encypt string to hex', required=False)
    parser.add_argument('-d','--decrypt', nargs=1, help='decypt hex to string', required=False)
    parser.add_argument('-cf','--create-cipherfile', action='store_true', help='create a new cipher file', required=False)

    args = vars(parser.parse_args())
    if args['encrypt']:
        os.system('cls')
        print(cipher(args['encrypt'][0]))

    if args['decrypt']:
        os.system('cls')        
        print(decipher(args['decrypt'][0]))      
        
    if args['create_cipherfile']:
        os.system('cls')        
        build_cipher_file()


# -*- coding: utf-8 -*-
"""
Created on Mon Nov 28 13:16:34 2016

@author: doarni
"""

import os

class openFile():
    def __init__(self):   
        self.path = os.path.dirname(os.path.realpath(__file__))
        self.loginData = None
        self.mainData = None
        self.caseData = None
        self.orderData = None
        self.orderpagelineData = None
        self.userpageData = None
        self.adjustmentsPageData = None
        self.htmlFiles = ['login.html','main.html', 'casepage.html', 'orderpage.html','orderpageline.html','userpage.html', 'adjustment.html']

    def loadFiles(self):
#        os.chdir('..')
#        os.chdir(self.path)
        with open(self.path + '\\login.html','r') as f:
            self.loginData = f.read().replace('\n','')
        with open(self.path + '\\main.html','r') as f:
            self.mainData = f.read().replace('\n','')
            
    def getLoginData(self):
        return self.loginData
        
    def getMainData(self):
        return self.mainData
        
    def getCaseData(self):
        with open(self.path + '\\casepage.html', 'r') as f:
            self.caseData = f.read().replace('\n','')
        return self.caseData
    
    def getOrderData(self):
        with open(self.path + '\\orderpage.html', 'r') as f:
            self.orderData = f.read().replace('\n','')
        return self.orderData
        
    def getOrderLineData(self):
        with open(self.path + '\\orderpageline.html', 'r') as f:
            self.orderpagelineData = f.read().replace('\n','')
        return self.orderpagelineData
        
    def getUserPageData(self):
        with open(self.path + '\\userpage.html', 'r') as f:
            self.userpageData = f.read().replace('\n','')
        return self.userpageData
        
    def getAdjustmentPageData(self):
        with open(self.path + '\\adjustment.html', 'r') as f:
            self.adjustmentsPageData = f.read().replace('\n','')
        return self.adjustmentsPageData
    
    def cleanUp(self):
        os.system("")
        
if __name__ == '__main__':
    filelist = ['login.html',
                'main.html',
                'orderpage.html',
                'orderpageline.html',
                'casepage.html',
                'userpage.html']
    for file in filelist:
        if os.path.isfile(file) == False:
            with open(file, 'w')as f:
                f.close()



stMutateSavebutton = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > span > div > div.GLNOMTPDN4C.GLNOMTPDK4C'
stMutateAlert = 'body > div.gwt-PopupPanel > div > div > table > tbody > tr > td > div > table > tbody > tr > td > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
stnote = 'CI-ZDI Mutation, Loaners Late Debit Policy'
stMutateCancelButton = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td.GLNOMTPDJYD > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
stMutateAllAvailable = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(3) > td.GLNOMTPDDS > div > span > img'
stMutateQuantity = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(2) > td.GLNOMTPDDS > input'
stMutateNote = 'body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div.GLNOMTPDNBD > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(4) > td.GLNOMTPDDS > textarea'
stMutateError = 'body > div:nth-child(13) > div > div > table > tbody > tr > td > div > table > tbody > tr > td > span > div > div.GLNOMTPDJ3C.GLNOMTPDG3C'
stMutateButton = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(7) > table > tbody > tr > td > span'



import os
import subprocess
import time
from pySMS.utils.excepetions import NetworkDriveError, NetworkError
from pySMS.pgsql.postgres import Postgres

DEFAULT_REMOTE_SHARE_NAME = r"\\nam\wardfs"
DEFAULT_DRIVE_LETTER = "G:"
DEFAULT_PROGRAM_LOCATION = r"G:\Dist\Centralized Inventory\ZB - Loaners Department\Ian"
VPN_HOST_IP_ADDRESS = "12.40.148.10"
POSTGRES_ADDRESS = "vsbslgprd01.zmr.zimmer.com"


class Protocols:

    def __init__(self):
        self.pg = Postgres()

    def test_connection(self, retry=5, address=POSTGRES_ADDRESS):
        for _ in range(retry):
            try:
                self.pg.test_connection(connection="MANUAL", username="doarni", password="ZimmerBiomet", host=address, verbose=False)
                return
            except:
                if _ < retry:
                    print('No connection established, retrying connection [' + str(_) + ']')
                    continue
                elif _ >= retry:
                    raise NetworkError("Please check to see if you are connected to the internet via VPN",
                                       issue="ServerConnectionFailed",
                                       msg="Connection to the server failed")

    @staticmethod
    def protocol_data():
        return {"DEFAULT_REMOTE_SHARE_NAME": DEFAULT_REMOTE_SHARE_NAME,
                "DEFAULT_DRIVE_LETTER": DEFAULT_DRIVE_LETTER,
                "DEFAULT_PROGRAM_LOCATION": DEFAULT_PROGRAM_LOCATION,
                "VPN_HOST_IP_ADDRESS": VPN_HOST_IP_ADDRESS,
                "POSTGRES_ADDRESS": POSTGRES_ADDRESS}


class NetworkDrive:

    def __init__(self):
        p = Protocols()
        p.test_connection()

        self.protocol_data = p.protocol_data()
        self.network_drive_path = self.check_for_map()


    def map_shared_drive(self, location=DEFAULT_REMOTE_SHARE_NAME, drive=DEFAULT_DRIVE_LETTER, retry=5, sleep=1):
        iter = 0
        for _ in range(retry  + 1):
            if not _ >= retry:
                try:
                    subprocess.check_call(["net", "use", drive, location])
                    print('Connection to network drive [' + location + '] established [' + drive + '].')
                    return {'return': 1, 'msg': 'successful_map'}
                except Exception as e:
                    iter += 1
                    time.sleep(sleep)
                    print('Mapping failed. Attempting re-map [' + str(_ + 1) + '] of [' + str(retry) + ']')
                    continue
            elif _ >= retry:
                raise NetworkDriveError(location + ' Could not be to map to ' + drive + ' after [' + str(iter) + '] retrys', issue="MappingFailed")

    def check_for_map(self):
        if os.path.exists(DEFAULT_PROGRAM_LOCATION) == True:
            print('Connection to network drive [' + DEFAULT_REMOTE_SHARE_NAME + '] established [' + DEFAULT_DRIVE_LETTER + '].')
            return DEFAULT_PROGRAM_LOCATION
        else:
            try:
                self.map_shared_drive()
            except:
                raise NetworkDriveError('Please check to see if the drive is mapped correctly and that you are connected to a ZimmerBiomet VPN / Internet.',issue='DriveNotFound', msg=None)

    def set_current_dir(self, _dir_):
        if not _dir_.__contains__(r'C:'):
            os.chdir(self.network_drive_path + _dir_)
            return os.getcwd()
        else:
            raise NetworkDriveError('Incorrect volume label. You must stay in the G:\ drive.', issue='IncorrectVolumeError')

    def clean_core_version(self, verbose=False):
        _path = self.set_current_dir(r"\SMSA\Python\program\core")
        fs = [f for f in os.listdir(_path) if f.endswith('')]

        for file in fs:
            if verbose != False:
                print('Removing [{}] from [{}]'.format(file, DEFAULT_DRIVE_LETTER + DEFAULT_REMOTE_SHARE_NAME))
                os.system("""del /s /q {}""".format(file))
            else:
                os.system("""del /s /q {}""".format(file))


    def get_files_from_directory(self, directory, abs_path=False, file_types=[], ignore=[]):
        _path = self.set_current_dir(directory)
        _files = [f for f in os.listdir(_path) if f.endswith('')]
        return_list = []
        if len(file_types) == 0:
            raise NetworkDriveError('No file types provided for file search', issue="NoFilesSpecified")
        else:
            for f in _files:
                if len(ignore) == 0:
                    if abs_path != False:
                        return_list.append(_path + '\\' + f)
                    else:
                        return_list.append(f)
                else:
                    ext = f.split('.')[1]
                    if ext not in ignore:
                        if abs_path != False:
                            return_list.append(_path + '\\' + f)
                        else:
                            return_list.append(f)
            return return_list



# -*- coding: utf-8 -*-
"""
Created on Wed Dec  7 12:30:48 2016

@author: doarni
"""


import xlrd
from pySMS.utils.logic.webpage import Webpage
from colorama import init, Fore


init(autoreset=True)

skipped = []

class parse():
    
    def readRows(filename, y, sheetIndex):

        if sheetIndex == 2:
            productNumber = None
            ediNumber = None
            description = None
            lotNumber = None
            serialNumber = None
            caseNumber = None
            _pass = False

            workbook = xlrd.open_workbook(filename)
            book = workbook.sheet_by_index(sheetIndex - 1)

            row = book.row_values(y + 1)
            tNumber = str(row[1])
            siteName = str(row[2])
            productNumber = str(row[7])
            ediNumber = str(row[8])
            description = str(row[9])
            lotNumber = str(row[10])
            serialNumber = str(row[11])
            caseNumber = str(row[12])
            _pass = False

            if lotNumber == '' and serialNumber == '':
                print(Fore.LIGHTRED_EX + productNumber + " at row " + str(y + 1) + " has no lot or serial! It was not added to the mutation queue")
                skipped.append(productNumber + ';' + str(y + 1))
                _pass = True
            else:
                print(Fore.LIGHTYELLOW_EX  +'Adding to queue: \tProduct: [{}] \t\tlot\serial: [{}]'.format(Fore.LIGHTCYAN_EX + productNumber + Fore.LIGHTYELLOW_EX, Fore.LIGHTCYAN_EX + (lotNumber if lotNumber != '' else serialNumber)) + Fore.LIGHTYELLOW_EX)

            return {'totalRows': book.nrows - 1,
                    'territoryNumber': tNumber,
                    'territoryName': siteName,
                    'productNumber': productNumber,
                    'ediNumber': ediNumber,
                    'description': description,
                    'lotNumber': lotNumber,
                    'serialNumber': serialNumber,
                    'caseNumber': caseNumber,
                    'bin?': "No",
                    "pass": _pass}

        if sheetIndex == 3:

            productNumber = None
            ediNumber = None
            description = None
            lotNumber = None
            serialNumber = None
            caseNumber = None
            bin = None
            _pass = False

            workbook = xlrd.open_workbook(filename)
            book = workbook.sheet_by_index(sheetIndex - 1)

            row = book.row_values(y + 1)
            tNumber = str(row[1])
            siteName = str(row[2])
            bin = str(row[5])
            productNumber = str(row[6])
            ediNumber = str(row[7])
            description = str(row[8])
            lotNumber = str(row[9])
            serialNumber = str(row[10])
            caseNumber = "None"
            _pass = False

            if lotNumber == '' and serialNumber == '':
                print(Fore.LIGHTRED_EX + productNumber + " at row " + str(y + 1) + " has no lot or serial! It was not added to the mutation queue")
                skipped.append(productNumber + ';' + str(y + 1))
                _pass = True
            else:
                print(Fore.LIGHTYELLOW_EX  +'Adding to queue: \tProduct: [{}] \t\tlot\serial: [{}]'.format(Fore.LIGHTCYAN_EX + productNumber + Fore.LIGHTYELLOW_EX, Fore.LIGHTCYAN_EX + (lotNumber if lotNumber != '' else serialNumber)) + Fore.LIGHTYELLOW_EX)

            return {'totalRows': book.nrows - 1,
                    'territoryNumber': tNumber,
                    'territoryName': siteName,
                    'productNumber': productNumber,
                    'ediNumber': ediNumber,
                    'description': description,
                    'lotNumber': lotNumber,
                    'serialNumber': serialNumber,
                    'caseNumber': caseNumber,
                    'binlocation': bin,
                    "bin?": "Yes",
                    "pass": _pass}



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 28 14:07:30 2016

@author: doarni
"""

try:
    import pySMS.html.get as get
    import pySMS.html.parse as parse
except:
    import get
    import parse



class stringBuilder():
    def buildStringPath(htmlData, path, index):
        return parse.parse.returnClassName(htmlData, path, index)

class buildPaths():
    
    class buildDataFromPath():
        def __init__(self):
            self.g = get.openFile()
            self.g.loadFiles()
            self.l = self.g.getLoginData()
            self.m = self.g.getMainData()
        
        def bClass(self, path, isLogin, isMain):
            if isLogin == True:
                return parse.parse.returnClassData(self.l, path)
            if isMain == True:
                return parse.parse.returnClassData(self.m, path)
        
    
    class buildCustom():
        
        def __init__(self):
            self.g = get.openFile()
            self.g.loadFiles()
            self.l = self.g.getLoginData()
            self.m = self.g.getMainData()
            
        def bLoginPageElement(self, path, indexNumber):
            return parse.parse.returnClassName(self.l, path, indexNumber)
        
        def bMainPageElement(self, path, indexNumber):
            return parse.parse.returnClassName(self.m, path, indexNumber)

    
    class buildLoginPaths():
        
        def __init__(self):
            self.g = get.openFile()
            self.g.loadFiles()
            self.l = self.g.getLoginData()

        
        def bUsername(self):
            path = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td > div'
            p1 = parse.parse.returnClassName(self.l, path, 0)
            path2 = path + '.' + p1 + ' > table > tbody > tr:nth-child(2) > td'
            p2 = parse.parse.returnClassName(self.l, path2, 1)
            return path2 + '.' + p2 + ' > input'
        
        def bPassword(self):
            path = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td > div'
            p1 = parse.parse.returnClassName(self.l, path, 0)
            path2 = path + '.' + p1 + ' > table > tbody > tr:nth-child(3) > td'
            p2 = parse.parse.returnClassName(self.l, path2, 1)
            return path2 + '.' + p2 + ' > input'
            
        def bLoginButton(self):
            path = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td > div'
            p1 = parse.parse.returnClassName(self.l, path, 0)
            path2 = path + '.' + p1 + ' > table > tbody > tr:nth-child(5) > td > table > tbody > tr > td > span > div > div'
            p2 = parse.parse.returnClassName(self.l, path2, 1)
            p2l = p2.split(' ')
            p2j = '.'.join(p2l)
            return path2 + '.' + p2j

    class buildMainPaths(object):
        
        def __init__(self):
            self.g = get.openFile()
            self.g.loadFiles()
            self.u = self.g.getUserPageData()
            self.m = self.g.getMainData()
            self.c = self.g.getCaseData()
            self.co = self.g.getOrderData()
            self.col = self.g.getOrderLineData()
            
            
        #Paths that do not require a class
        def bStockTab(self):
            return "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(2) > table > tbody > tr > td:nth-child(1) > table > tbody > tr > td:nth-child(2) > div"
        
        #Paths that require classes
        def bCheckSite(self):
            path = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td'
            p1 = parse.parse.returnClassName(self.m, path, 0)
            path2 = path + '.' + p1 + ' > table > tbody > tr > td'
            p2 = parse.parse.returnClassName(self.m, path2, 1)
            return path2 + '.' + p2 + ' > span'
            
        def bCaseTab(self):
            path = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td'
            p1 = parse.parse.returnClassName(self.m, path, 1)
            return path + '.' + p1 + ' > img:nth-child(1)'
        
        def bInventoryTab(self):
            path = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td'
            p1 = parse.parse.returnClassName(self.m, path, 1)
            return path + '.' + p1 + ' > img:nth-child(2)'
            
    #Adjustments Page
    class buildAdjustments():
        
        def __init__(self):
            self.g = get.openFile()
            self.adj = self.g.getAdjustmentPageData()
            
        def adjAdjustButton(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td"
            p1 = parse.parse.returnClassName(self.adj, path, 1)
            path2 = path + '.' + p1 + " > table > tbody > tr > td"
            p2 = parse.parse.returnClassName(self.adj, path2, 9)
            return path2 + '.' + p2 + " > span"

            
        def adjMenuQuantity(self):
            path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
            p1 = parse.parse.returnClassName(self.adj, path, 1)
            path2 = path + '.' + p1 + " > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(1) > td"
            p2 = parse.parse.returnClassName(self.adj, path2, 1)
            return path2 + '.' + p2 + " > input"
            
        def adjMenuNote(self):
            path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
            p1 = parse.parse.returnClassName(self.adj, path, 1)
            path2 = path + '.' + p1 + " > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(2) > td"
            p2 = parse.parse.returnClassName(self.adj, path2, 1)
            return path2 + '.' + p2 + " > textarea"
            
        def adjMenuCancelButton(self):
            path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
            p1 = parse.parse.returnClassName(self.adj, path, 1)
            path2 = path + '.' + p1 + " > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td"
            p2 = parse.parse.returnClassName(self.adj, path2, 0)
            path3 = path2 + '.' + p2 + " > span > div > div"
            p3 = parse.parse.returnClassName(self.adj, path3, 1)
            return path3 + "." + '.'.join(p3.split(' ')) 
            
        def adjMenuSaveButton(self):    
            path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
            p1 = parse.parse.returnClassName(self.adj, path, 1)
            path2 = path + '.' + p1 + " > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > span > div > div"
            p2 = parse.parse.returnClassName(self.adj, path2, 1)
            return path2 + '.' + '.'.join(p2.split(' ')) 
            
    #User Page
    class buildUserPage(buildMainPaths):
        def __init__(self):
            super(buildPaths.buildUserPage, self).__init__()
            
        def uUser(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > table > tbody > tr > td.GLNOMTPDLX > table > tbody > tr > td"
            p1 = parse.parse.returnClassName(self.u, path, 0)
            return path + '.' + p1 + " > span"
        
        def uUserSettingsButton(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div > div > div > div"
            p1 = parse.parse.returnClassName(self.u, path, 0)
            path2 = path + '.' + p1 + " > table > tbody > tr:nth-child(1) > td:nth-child(2) > table > tbody > tr > td"
            p2 = parse.parse.returnClassName(self.u, path2, 2)
            return path2 + '.' + p2 + " > span"
            
        def uUserSettingsMenu(self):
            def saveButton():
                path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
                p1 = parse.parse.returnClassName(self.u, path, 1)
                path2 = path + '.' + p1 + " > div > div > table > tbody > tr:nth-child(8) > td > table > tbody > tr > td:nth-child(2) > span > div > div"
                p2 = parse.parse.returnClassName(self.u, path2, 1)
                return path2 + "." + '.'.join(p2.split(' '))
            
            def listBoxSites():
                dict = {}
                path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
                p1 = parse.parse.returnClassName(self.u, path, 1)
                path2 = path + '.' + p1 + " > div > div > table > tbody > tr:nth-child(1) > td"
                p2 = parse.parse.returnClassName(self.u, path2, 1)
                for i in range(52, 108):
                    path3 = path2 + '.' + p2 + " > select > option:nth-child(" + str(i + 1) + ")"
                    value = str(parse.parse.returnTextInfo(self.u, path3)[1])
                    name = (parse.parse.returnTextInfo(self.u, path3)[0]).split(' ')
                    name = ('_'.join(name)).rstrip('_' + name[-1])
                    if 'OSTEO' in name:
                        pass
                    else:
                        dict[name] = {'value': value, 'path': path3}
                return dict

            dict = {'listBoxSites': listBoxSites()}
            dict['listBoxSites']['saveButton'] = {'value': '0', 'path': saveButton()}

            return dict
          

    #StockTab
    class buildStockTab(buildMainPaths):
        def __init__(self):
            super(buildPaths.buildStockTab, self).__init__()
       
        def stStockTabButton(self):
            return "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(2) > table > tbody > tr > td:nth-child(1) > table > tbody > tr > td:nth-child(2) > div"
        
        def stStockTabSearchButton(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(1) > span > div > div"
            p1 = parse.parse.returnClassName(self.m, path, 1)
            return path + "." + '.'.join(p1.split(' '))
            
        def stStockTabResults(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(1) > div > table > tbody > tr > td > table > tbody > tr > td"
            p1 = parse.parse.returnClassName(self.m, path, 1)
            return path + "." + p1
            
        #Stock Tab Table
       
        #Checkboxes, because they are a pain in the ass, classes are different on odd and even rows
        def stTable_CheckBox_Init(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(1) > td"
            p1 = parse.parse.returnClassName(self.m, path, 0)
            p1j = '.'.join(p1.split(' '))          
            return path + "." + p1j + " > div > span > img"
        
        def stTable_CheckBox_Row_Even(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(2) > td"
            p1 = parse.parse.returnClassName(self.m, path, 0)
            p1j = '.'.join(p1.split(' '))  
            pathFront = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child("
            pathBack = ") > td." + p1j + " > div > span > img"
            return [pathFront, pathBack]
            
        def stTable_CheckBox_Row_Odd(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(3) > td"
            p1 = parse.parse.returnClassName(self.m, path, 0)
            p1j = '.'.join(p1.split(' '))         
            pathFront = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child("
            pathBack = ") > td." + p1j + " > div > span > img"            
            return [pathFront, pathBack]
            
        #Table Data
        def stTableData(self):
            #Returns a dictionary of split css seletors which are meant to be joined with a row number in between. Ex; dict['product'][0] + str(number) + dict['product'][1]
            def dataDict():
                
                dict = {'product': ["body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(",
                            ") > td:nth-child(2) > span"],
                        'description': ["body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(",
                                        ") > td:nth-child(3) > div"],
                        'lot_serial': ["body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(",
                                       ") > td:nth-child(4) > div"],
                        'QOH': ["body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(",
                                ") > td:nth-child(7) > div"],
                        'QAV': ["body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(",
                                ") > td:nth-child(8) > div"],
                        'QR': ["body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(",
                               ") > td:nth-child(9) > div"],
                        'ZSMS_hold': zsms_Data(),
                        'location': location_Data(),
                        'contained_in': conatainedIn_Data()}  
                
                return dict
            
            def conatainedIn_Data():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(1) > td:nth-child(5) > div > table > tbody > tr > td"
                
                p1 = parse.parse.returnClassName(self.m, path, 1)
                p2 = parse.parse.returnClassName(self.m, path, 0)
                
                pathFront = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child("
                pathBackInfo = ") > td:nth-child(5) > div > table > tbody > tr > td" + "." + p1 + " > span"
                pathBackType = ") > td:nth-child(5) > div > table > tbody > tr > td" + "." + p2
                pathInit = path + "." + p1
                return {'init': pathInit, 'front': pathFront, 'backInfo': pathBackInfo, 'backType': pathBackType}
            
            def location_Data():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(1) > td:nth-child(6) > div > table > tbody > tr > td"
 
                p1 = parse.parse.returnClassName(self.m, path, 0)
                p2 = parse.parse.returnClassName(self.m, path, 1)
                pathFront = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(" 
                pathBackInfo = ") > td:nth-child(6) > div > table > tbody > tr > td" + "." + p2 + " > span"
                pathBackLocType = ") > td:nth-child(6) > div > table > tbody > tr > td" + "." + p1
                return {'front': pathFront, 'backInfo': pathBackInfo, 'backLocType': pathBackLocType}
             
            def zsms_Data():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(1) > td"
                path2 = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(2) > td"
                path3 = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(3) > td"
                
                p1 = parse.parse.returnClassName(self.m, path, 9)
                p2 = parse.parse.returnClassName(self.m, path2, 9)
                p3 = parse.parse.returnClassName(self.m, path3, 9)
                
                pathInit = path + "." + '.'.join(p1.split(' ')) + " > div"
                pathFront = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child("
                pathBackEven = ") > td" + "." + p2 + " > div"
                pathBackOdd = ") > td" + "." + p3 + " > div"
                return {'init': pathInit, 'front': pathFront, 'backEven': pathBackEven, 'backOdd': pathBackOdd}
            
            return dataDict()
            
        
        def stMutate(self):
            
            def mutateButton():
                return "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td:nth-child(7) > table > tbody > tr > td > span"
                        
            def mutateQuantityBox():
                path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
                p1 = parse.parse.returnClassName(self.m, path, 1)
                path2 = path + "." + p1 + " > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(2) > td"
                p2 = parse.parse.returnClassName(self.m, path2, 1)
                return path2 + "." + p2 + " > input"
            
            def mutateNoteBox():
                path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
                p1 = parse.parse.returnClassName(self.m, path, 1)
                path2 = path + "." + p1 + " > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(4) > td"
                p2 = parse.parse.returnClassName(self.m, path2, 1)
                return path2 + "." + p2 + " > textarea"
            
            def mutateAllAvailableCheckBox():
                path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
                p1 = parse.parse.returnClassName(self.m, path, 1)
                path2 = path + "." + p1 + " > div > table > tbody > tr:nth-child(1) > td > table > tbody > tr:nth-child(3) > td"
                p2 = parse.parse.returnClassName(self.m, path2, 1)
                return path2 + "." + p2 + " > div > span > img"
            
            def mutateSaveButton():
                path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
                p1 = parse.parse.returnClassName(self.m, path, 1)
                path2 = path + "." + p1 + " > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > span > div > div"
                p2 = parse.parse.returnClassName(self.m, path2, 1)
                return path2 + "." + '.'.join(p2.split(' '))
                
            def mutateCancelButton():
                path = "body > div.gwt-PopupPanel > div > div > div > table > tbody > tr > td > div > div"
                p1 = parse.parse.returnClassName(self.m, path, 1)
                path2 = path + "." + p1 + " > div > table > tbody > tr:nth-child(2) > td > table > tbody > tr > td"
                p2 = parse.parse.returnClassName(self.m, path2, 0)
                path3 = path2 + "." + p2 + " > span > div > div"
                p3 = parse.parse.returnClassName(self.m, path3, 1)
                return path3 + "." + '.'.join(p3.split(' '))
                
            def mutateAlert():
                path = "body > div.gwt-PopupPanel > div > div > table > tbody > tr > td > div > table > tbody > tr > td > span > div > div"
                p1 = parse.parse.returnClassName(self.m, path, 1)
                return path + "." + '.'.join(p1.split(' '))
                
            def mutateError():
                path = "body > div:nth-child(13) > div > div > table > tbody > tr > td > div > table > tbody > tr > td > span > div > div"
                p1 = parse.parse.returnClassName(self.m, path, 1)
                return path + "." + '.'.join(p1.split(' '))

            def mutateMenu():
                dict = {'mutateButton': mutateButton(),
                        'mutateQuantity': mutateQuantityBox(),
                        'mutateNote': mutateNoteBox(),
                        'mutateAllAvailable': mutateAllAvailableCheckBox(),
                        'mutateSaveButton': mutateSaveButton(),
                        'mutateCancelButton': mutateCancelButton(),
                        'mutateAlert': mutateAlert(),
                        'mutateError': mutateError()}
                
                return dict
                
            return mutateMenu()
        
        #Case Page
        def cDistribution(self):
            
            def paybackOrders():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(8) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > table > tbody > tr > td"
                p1 = parse.parse.returnClassName(self.c, path, 1)
                path2 = path + '.' + p1 + " > div:nth-child(1) > div"
                p2 = parse.parse.returnClassName(self.c, path2, 1)
                path3 = path2 + '.' + p2 + " > table > tbody > tr > td > table > tbody > tr:nth-child(5) > td"
                p3 = parse.parse.returnClassName(self.c, path3, 1)
                return path3 + '.' + p3 + " > table > tbody > tr > td > span"
                
            dict = {'cDistributionPaybackOrders': paybackOrders()}
            
            return dict
        
        #Case Page, Payback Order
        def cpbTableData(self):
            
            
            def checkBoxes():
                def initPath():
                    path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                    p1 = parse.parse.returnClassName(self.co, path, 2)
                    path2 = path + '.' + p1 + " > div > div"
                    p2 = parse.parse.returnClassName(self.co, path2, 1)
                    path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                    p3 = parse.parse.returnClassName(self.co, path3, 1)
                    return path3 + '.' + p3

                def firstRow():
                    path = initPath() + " > div > table > tbody > tr:nth-child(1) > td"
                    p1 = parse.parse.returnClassName(self.co, path, 0)
                    return path + '.' + '.'.join(p1.split(' ')) + " > div > span > img"
                    
                def evenRow():
                    path = initPath() + " > div > table > tbody > tr:nth-child(2) > td"
                    p1 = parse.parse.returnClassName(self.co, path, 0)
                    front = initPath() + " > div > table > tbody > tr:nth-child("
                    back = ") > td" + '.' + p1 + " > div > span > img"
                    return {'front': front, 'back': back}
                    
                def oddRow():
                    path = initPath() + " > div > table > tbody > tr:nth-child(3) > td"
                    p1 = parse.parse.returnClassName(self.co, path, 0)
                    front = initPath() + " > div > table > tbody > tr:nth-child("
                    back = ") > td" + '.' + p1 + " > div > span > img"
                    return {'front': front, 'back': back}
                    
                dict = {'Init': firstRow(),
                        'Even': evenRow(),
                        'Odd': oddRow()}
                
                return dict
            
            def refNumber():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                p1 = parse.parse.returnClassName(self.co, path, 2)
                path2 = path + '.' + p1 + " > div > div"
                p2 = parse.parse.returnClassName(self.co, path2, 1)
                path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                p3 = parse.parse.returnClassName(self.co, path3, 1)
                front = path3 + '.' + p3 + " > div > table > tbody > tr:nth-child("
                back = ") > td:nth-child(2) > span"
                return {'Front': front,'Back': back}

            def product():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                p1 = parse.parse.returnClassName(self.co, path, 2)
                path2 = path + '.' + p1 + " > div > div"
                p2 = parse.parse.returnClassName(self.co, path2, 1)
                path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                p3 = parse.parse.returnClassName(self.co, path3, 1)
                front = path3 + '.' + p3 + " > div > table > tbody > tr:nth-child("
                back = ") > td:nth-child(3) > span"
                return {'Front': front,'Back': back}
                
            def description():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                p1 = parse.parse.returnClassName(self.co, path, 2)
                path2 = path + '.' + p1 + " > div > div"
                p2 = parse.parse.returnClassName(self.co, path2, 1)
                path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                p3 = parse.parse.returnClassName(self.co, path3, 1)
                front = path3 + '.' + p3 + " > div > table > tbody > tr:nth-child("
                back = ") > td:nth-child(4) > div"
                return {'Front': front,'Back': back}
            
            def qtyRequested():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                p1 = parse.parse.returnClassName(self.co, path, 2)
                path2 = path + '.' + p1 + " > div > div"
                p2 = parse.parse.returnClassName(self.co, path2, 1)
                path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                p3 = parse.parse.returnClassName(self.co, path3, 1)
                front = path3 + '.' + p3 + " > div > table > tbody > tr:nth-child("
                back = ") > td:nth-child(6) > div"
                return {'Front': front,'Back': back}

            def qtyOpen():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                p1 = parse.parse.returnClassName(self.co, path, 2)
                path2 = path + '.' + p1 + " > div > div"
                p2 = parse.parse.returnClassName(self.co, path2, 1)
                path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                p3 = parse.parse.returnClassName(self.co, path3, 1)
                front = path3 + '.' + p3 + " > div > table > tbody > tr:nth-child("
                back = ") > td:nth-child(7) > div"
                return {'Front': front,'Back': back}

            def qtyReserved():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                p1 = parse.parse.returnClassName(self.co, path, 2)
                path2 = path + '.' + p1 + " > div > div"
                p2 = parse.parse.returnClassName(self.co, path2, 1)
                path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                p3 = parse.parse.returnClassName(self.co, path3, 1)
                front = path3 + '.' + p3 + " > div > table > tbody > tr:nth-child("
                back = ") > td:nth-child(8) > div"
                return {'Front': front,'Back': back}

            def qtyOnMovement():
                path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(5) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div"
                p1 = parse.parse.returnClassName(self.co, path, 2)
                path2 = path + '.' + p1 + " > div > div"
                p2 = parse.parse.returnClassName(self.co, path2, 1)
                path3 = path2 + '.' + p2 + " > div > div > div:nth-child(2) > div"
                p3 = parse.parse.returnClassName(self.co, path3, 1)
                front = path3 + '.' + p3 + " > div > table > tbody > tr:nth-child("
                back = ") > td:nth-child(9) > div"
                return {'Front': front,'Back': back}
            
            dict = {'cpbTableDataCheckBoxes': checkBoxes(),
                    'cpbTableDataRefNumber': refNumber(),
                    'cpbTableDataProductNumber': product(),
                    'cpbTableDataDescription': description(),
                    'cpbTableDataQTYRequested': qtyRequested(),
                    'cpbTableDataQTYOpen': qtyOpen(),
                    'cpbTableDataQTYReserved': qtyReserved(),
                    'cpbTableDataQTYOnMovement': qtyOnMovement()}
            
            return dict

            
        #Product Chooser
        def pcSelect(self):
            path = "body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > div > div:nth-child(1) > div:nth-child(2) > div"
            p1 = parse.parse.returnClassName(self.m, path, 1)            
            return path + '.' + p1 + " > table > tbody > tr > td > table > tbody > tr > td:nth-child(2) > span"
                
        def pcSearchBar(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr:nth-child(1) > td"
            p2 = parse.parse.returnClassName(self.m, path2, 0)
            return path2 + "." + p2 + " > input"
        
        def pcSearchButton(self):
            path = "body > div"
            p0 = parse.parse.returnClassName(self.m, path, 3)
            path0 = path + "." + p0 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr:nth-child(1) > td"
            p1 = parse.parse.returnClassName(self.m, path0, 1)
            path2 = path0 + '.' + p1 + "  > span > div > div"
            p2 = parse.parse.returnClassName(self.m, path2, 1)
            p2j = '.'.join(p2.split(' '))
            return path2 + '.' + p2j
            
        def pcFinishButton(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > table > tbody > tr > td"
            p2 = parse.parse.returnClassName(self.m, path2, 1)
            path3 = path2 + "." + p2 + " > span > div > div"
            p3 = parse.parse.returnClassName(self.m, path3, 1)
            p3j = '.'.join(p3.split(' '))
            return path3 + '.' + p3j

        def pcTable(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody"
            return path2

        def pcAdd(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr > td"
            p2 = parse.parse.returnClassName(self.m, path2, 0)
            p2j = '.'.join(p2.split(' '))
            path3 = path2 + "." + p2j + " > div > table > tbody > tr > td > table > tbody > tr > td"
            p3 = parse.parse.returnClassName(self.m, path3, 0)
            return path3 + "." + p3 + " > span"

        def pcAddLast(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(6) > td"
            p2 = parse.parse.returnClassName(self.m, path2, 0)
            p2j = '.'.join(p2.split(' '))
            path3 = path2 + "." + p2j + " > div > table > tbody > tr > td > table > tbody > tr > td"
            p3 = parse.parse.returnClassName(self.m, path3, 0)
            front = path + "." + p1 + "> div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(" \
                                      "2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) " \
                                      "> div > div:nth-child(3) > div > div > table > tbody > tr:nth-child( "
            back = ") > td" + "." + p2j + " > div > table > tbody > tr > td > table > tbody > tr > td" + "." + p3 + " > span"
            return [front, back]

        def pcAddFront(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child("
            return path2 
        
        def pcAddOddBack(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(2) > td"
            p2 = parse.parse.returnClassName(self.m, path2, 0)
            p2j = '.'.join(p2.split(' '))
            path3 = path2 + "." + p2j + " > div > table > tbody > tr > td > table > tbody > tr > td"
            p3 = parse.parse.returnClassName(self.m, path3, 0)
            return ") > td" + "." + p2j + " > div > table > tbody > tr > td > table > tbody > tr > td" + "." + p3 + " > span"
        
        def pcAddEvenBack(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(3) > td"
            p2 = parse.parse.returnClassName(self.m, path2, 0)
            p2j = '.'.join(p2.split(' '))
            path3 = path2 + "." + p2j + " > div > table > tbody > tr > td > table > tbody > tr > td"
            p3 = parse.parse.returnClassName(self.m, path3, 0)
            return ") > td" + "." + p2j + " > div > table > tbody > tr > td > table > tbody > tr > td" + "." + p3 + " > span"
            
        def pcTableData(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            front = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child("
            back = ") > td:nth-child(3) > div"
            return [front, back]
            
        def pcResults(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            path2 = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(2) > div > table > tbody > tr > td > table > tbody > tr > td"
            p2 = parse.parse.returnClassName(self.m, path2, 1)
            return path2 + "." + p2

        def pcTRPaths(self):
            path = "body > div"
            p1 = parse.parse.returnClassName(self.m, path, 3)
            front = path + "." + p1 + " > div > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(4) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child("
            back = ")"
            return [front, back]
            
                
# b = buildPaths()
# bl = b.buildLoginPaths()
# print(bl.bLoginFailed())


# -*- coding: utf-8 -*-
"""
Created on Thu Nov 17 14:08:50 2016

@author: doarni
"""
import os
import time
import json
import configparser
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium import *
from selenium.webdriver.common.action_chains import ActionChains
from pySMS.livery import tables as lv
from pySMS.prompts import prompts as pmt
from pySMS.ui.tk import widget
from pySMS.handlers import Handles
from pySMS.utils.logger import Logger
from pySMS.utils.logic.webpage import Webpage
from pySMS.utils.excepetions import NoRecordsFoundError, WaitTimeOutError, ProductChooserAddError, MissingInformationError
import pySMS.html.pathdata.loginPage as loginpage
import pySMS.html.pathdata.mainPage as mainpage
import pySMS.html.pathdata.prodChooser as stprodchooser
import pySMS.html.pathdata.stockTabMutate as stmutate
import pySMS.html.pathdata.stockTabTable as sttable
import pySMS.html.pathdata.stockTab as st
import pySMS.html.pathdata.userSettings as uS
import pySMS.html.pathdata.adjustStock as adj
from colorama import init, Fore

init(autoreset=True)


class macros():
    
    def __init__(self, name, pause_at_launch=False):
        self.name = name
        self.pause_at_launch= pause_at_launch
        self.driver = None
        self.path = os.path.dirname(os.path.realpath(__file__))

        self.web_logic = Webpage.Logic()
        self.w = widget()
        self.handles = Handles()
        self.err = self.handles.Errors()
        self.jdata = self.handles.Data()
        self.logger = Logger(self.name)

        self.log_list_info = []
        self.log_list_error = []
        self.log_list_warning = []
        self.log_list_critical = []
        self.log_list_debug = []

        self.logininfo = []
        self.current_func_data = {}

        self.actions = None

        self.siteData = self.jdata.load_json_data()

        self.current_error_count = 0
        self.adjustmentErrCount = 0

    def parse_settings_for(self, section, value):
        config = configparser.ConfigParser()
        config.read('settings.ini')
        try:
            return config.get(section, value)
        except:
            return None

    def check_for_login_information(self):
        username = self.parse_settings_for('USER', 'username')
        password = self.parse_settings_for('USER', 'password')

        if username.upper() == 'NONE' and password.upper() == 'NONE':
            return {'exists': False, 'username': username, 'password': password}
        else:
            return {'exists': True, 'username': username, 'password': password}

    def log(self, _type, text):
        try:
            if _type.lower() == 'info':
                self.log_list_info.append(text)

            if _type.lower() == 'error':
                self.log_list_error.append(text)

            if _type.lower() == 'warning':
                self.log_list_warning.append(text)

            if _type.lower() == 'critical':
                self.log_list_critical.append(text)

            if _type.lower() == 'debug':
                self.log_list_debug.append(text)

            self.current_func_data['info'] = self.log_list_info
            self.current_func_data['error'] = self.log_list_error
            self.current_func_data['warning'] = self.log_list_warning
            self.current_func_data['critical'] = self.log_list_critical
            self.current_func_data['debug'] = self.log_list_debug

        except Exception as e:
            pass

    def write_log(self):
        try:
            self.logger.write_log_file(self.current_func_data)
        except Exception as e:
            pass

    def reportMutationErrors(self):
        return self.current_error_count

    def slay(self):
        print(Fore.RED + 'Disconnecting From SMS')
        self.log('info', '[INFO] Shutting down Webdriver')
        self.driver.quit()
        print(Fore.RED + 'Disconnected')
        self.log('info', '[INFO] Webdriver closed')

    def var(self): #must be called var, driver is not defined till launch call
        self.css = self.driver.find_element_by_css_selector

    def _wait(self, n):
        time.sleep(n)

    def launch(self):
        if self.check_for_login_information()['exists'] == False:
            raise MissingInformationError('user', ['username', 'password'], msg="Missing login information. Please configure this in settings.")
        else:
            chromedriver = self.path + "\\chromedriver.exe"
            os.environ["webdriver.chrome.driver"] = chromedriver
            options = webdriver.ChromeOptions()
            options.add_experimental_option("excludeSwitches", ["ignore-certificate-errors"])
            self.driver = webdriver.Chrome(chromedriver)
            print('SESSION_URL = ' + self.driver.command_executor._url)
            print('SESSION_ID = ' + self.driver.session_id)
            self.driver.maximize_window()
            self.driver.get(loginpage.url)
        if self.pause_at_launch != False:
            self.err.err_basic("Please press 'OK' when the page has fully loaded!", "SMSA")

    def hook_launch(self, url, session_id):
        if self.check_for_login_information()['exists'] == False:
            raise MissingInformationError('user', ['username', 'password'], msg="Missing login information. Please configure this in settings.")
        chromedriver = self.path + "\\chromedriver.exe"
        os.environ["webdriver.chrome.driver"] = chromedriver
        options = webdriver.ChromeOptions()
        options.add_experimental_option("excludeSwitches", ["ignore-certificate-errors"])
        self.driver = webdriver.Remote(command_executor=url, desired_capabilities=options.to_capabilities())
        self.driver.close()
        self.driver.session_id = session_id
        print('SESSION_URL = ' + self.driver.command_executor._url)
        print('SESSION_ID = ' + self.driver.session_id)
        self.driver.maximize_window()
        self.driver.get(loginpage.url)

    def input_to_element(self, selector, text):
        self.actions = ActionChains(self.driver)
        self.search = self.css(selector)
        self.search.click()
        self.clickinto = self.actions.move_to_element(self.search).send_keys(text + Keys.RETURN).perform()
        self.actions = None

    def get_inner_html(self, selector):
        return self.css(selector).get_attribute('innerHTML')

    def getHTMLLoginPage(self):
        print(Fore.YELLOW + 'Grabbing html info from login page')
        time.sleep(1)
        with open(self.path + '//html//login.html', 'w')as f:
            f.write(self.driver.page_source)
        f.close()
        print(Fore.YELLOW + 'Done.')
        print(Fore.YELLOW + 'Saved as login.html')

    def getHTML(self, filename):
        print(Fore.YELLOW + 'Grabbing html info from page')
        time.sleep(1)
        with open(self.path + '//html//' + filename,'w')as f:
            f.write(self.driver.page_source)
        f.close()
        print(Fore.YELLOW + 'Done.')
        print(Fore.YELLOW + 'Saved as ' + filename)
    
    def getHTMLMainPage(self):
        # used to grab HTML info from the site after logging in
        print(Fore.YELLOW + 'Grabbing html info from page')
        time.sleep(1)
        with open(self.path + '//html//main.html','w')as f:
            f.write(self.driver.page_source)
        f.close()
        print(Fore.YELLOW + 'Done.')
        print(Fore.YELLOW + 'Saved as main.html')
    
    def login(self):  
        print(Fore.YELLOW + 'Logging in as ' + self.parse_settings_for('USER', 'username'))
        self.log('info', '[INFO] [DEBUG] Logging in as ' + self.parse_settings_for('USER', 'username'))
        try:
            self.css(loginpage.username).clear()
            time.sleep(0.5)
            self.css(loginpage.username).send_keys(self.parse_settings_for('USER', 'username'))
            self.css(loginpage.password).clear()
            time.sleep(0.5)
            self.css(loginpage.password).send_keys(self.parse_settings_for('USER', 'password'))
            time.sleep(1)
            try:
                self.css(loginpage.loginbutton).click()
                print(Fore.YELLOW + 'Logged in successfully as ' + self.parse_settings_for('USER', 'username'))
                self.log('info', '[INFO] [DEBUG] Logged in successfully as ' + self.parse_settings_for('USER', 'username'))
            except Exception as _e:
                print(Fore.LIGHTRED_EX + 'Logged in Failed')
                print(Fore.LIGHTMAGENTA_EX + 'Traceback: ' + Fore.LIGHTRED_EX + str(_e))
                self.err.err_custom('Unable to login as ' + self.parse_settings_for('USER', 'username') + '. Please refresh the page and login manually.', 'Error', 'MB_OK', 'ICON_EXCLAIM')
                self.log('info', '[INFO] [ERROR] Unable to login as ' + self.parse_settings_for('USER', 'username') + '.')
                self.log('warning', str(_e))
        except Exception as e:
            print(Fore.LIGHTRED_EX + 'Logged in Failed')
            print(Fore.LIGHTMAGENTA_EX + 'Traceback: ' + Fore.LIGHTRED_EX + str(e))
            self.err.err_custom('Unable to login as ' + self.parse_settings_for('USER', 'username') + '. Please refresh the page and login manually.', 'Error', 'MB_OK', 'ICON_EXCLAIM')
            self.log('info', '[INFO] [ERROR] Unable to login as ' + self.parse_settings_for('USER', 'username') + '.')
            self.log('warning', str(e))

    def checksite(self):
        time.sleep(2)
        try:
            site = self.css(mainpage.checkSite).get_attribute('innerHTML')
            print(Fore.YELLOW + 'Logged into site ' + site + '\n')
            self.log('info', '[INFO] [DEBUG] Logged into site ' + site)
            return site
        except Exception as e:
            self.err.err_traceback(e,'Unable to check site you are logged in under, please verify this before continuing')
            self.log('info', '[INFO] [ERROR] Unable to check site you are logged in under, please verify this before continuing')
            self.log('warning', str(e))

    def change_site_strict(self, site):
        site_name = ('_'.join(site.split(' ')))
        for key, value in self.siteData['listBoxSites'].items():
            if site_name == str(key):
                self.driver.refresh()
                time.sleep(2)
                self.wait_for(loginpage.username)
                self.login()

                self.wait_for_click(uS.upUser)

                self.wait_for_click(uS.upUserSettingsButton)

                self.wait_for_click(self.siteData['listBoxSites'][str(key)]['path'])

                self.css(uS.upSaveButton).click()
                time.sleep(2)
                self.driver.refresh()
                time.sleep(2)
                self.wait_for(loginpage.username)
                self.login()

            if self.checksite == site:
                print(Fore.LIGHTYELLOW_EX + 'Successfully changed site to: ' + Fore.MAGENTA + site)
                self.log('info', '[INFO] [DEBUG] Successfully changed site to: ' + site)


    def change_site(self, site):
        site_name = ('_'.join(site.split(' ')))
        for key, value in self.siteData['listBoxSites'].items():
            if site_name == str(key):
                self.driver.refresh()
                time.sleep(2)
                self.wait_for(loginpage.username)
                self.login()

                self.wait_for_click(uS.upUser)

                self.wait_for_click(uS.upUserSettingsButton)

                self.wait_for_click(self.siteData['listBoxSites'][str(key)]['path'])

                self.css(uS.upSaveButton).click()
                time.sleep(2)
                self.driver.refresh()
                time.sleep(2)
                self.wait_for(loginpage.username)
                self.login()
                self.inventory_tab()
                self.stStockTabButton()

            if self.checksite == site:
                print(Fore.LIGHTYELLOW_EX + 'Successfully changed site to: ' + Fore.MAGENTA + site)
                self.log('info', '[INFO] [DEBUG] Successfully changed site to: ' + site)

    def testClick(self, css_selector):
        self.css(css_selector).click()

    def inventory_tab(self):
        try:
            self.wait_for_click(mainpage.InventoryTab)
        except Exception as e:
            self.err.err_basic('Unable to click Ribbon Button: Inventory Tab, please click the Ribbon Button: Inventory Tab then press ok to continue', 'Error')
            self.log('info', '[INFO] [ERROR] Unable to click on Ribbon Button: Inventory Tab'), self.log('warning', str(e))
            self.current_error_count += 1

    def stStockTabButton(self):
        try:
            self.wait_for_click(st.stStockTabButton)
        except Exception as e:
            self.err.err_basic('Unable to click Stock Tab button, please click the stock tab button then press ok to continue', 'Error')
            self.log('info', '[INFO] [ERROR] Unable to click on Stock Tab button'), self.log('warning', str(e))
            self.current_error_count += 1

    def stSearchButton(self):
        try:
            self.wait_for_click(st.stStockTabSearchButton)
        except Exception as e:
            self.err.err_basic('Unable to click search button, please click search button and click ok to continue', 'Error')
            self.log('info', '[INFO] [ERROR] Unable to click search button'), self.log('warning', str(e))
            self.current_error_count += 1

    def stProdChooserSelect(self):
        try:
            self.wait_for_click(stprodchooser.prodchooserSelect)
        except Exception as e:
            self.err.err_basic('Unable to open product chooser on stock page, please open and click ok to continue', 'Error')
            self.log('info', '[INFO] [ERROR] Unable to open product chooser on stock page'), self.log('warning', str(e))
            self.current_error_count += 1
            
    def stProdChooserSearch(self, item):
        try:
            self.actions = ActionChains(self.driver)
            self.search = self.css(stprodchooser.prodchooserSearchBar)
            self.search.click()
            self.clickinto = self.actions.move_to_element(self.search).send_keys(item + Keys.RETURN).perform()
            self.actions = None
        except Exception as e:
            self.err.err_basic('Unable to search for ' + item + ', please search and click ok to continue', 'Error')
            self.log('info', '[INFO] [ERROR] Unable to search for ' + item), self.log('warning', str(e))
            self.current_error_count += 1

    def stProdChooserAddItem(self, item_dict):
        def read_table(y_range):
            data_dict = {}
            try:
                for y in range(y_range[0], y_range[1] + 1):
                    for tr in self.driver.find_elements_by_css_selector(stprodchooser.prodChooserTRpathFront + str(y) + stprodchooser.prodChooserTRpathBack):
                        tds = tr.find_elements_by_tag_name('td')
                        if tds:
                            l = []
                            for td in tds:
                                try:
                                    l.append(td.text)
                                except:
                                    l.append('None')
                            data_dict['row' + str(y)] = l
                return data_dict
            except Exception as j:
                print(str(j))
        try:
            results_dict = self.web_logic.product_chooser_add_item(item_dict, read_table(self.getResults(stprodchooser.prodChooserResults)))
        except ProductChooserAddError as e:
            print("Unable to add item from product chooser! Item: " + item_dict['prodNumber'])
        try:
            for k, v in results_dict.items():
                if results_dict['action'] == 'none':
                    self.err.err_basic(results_dict['reason'], 'MatchError')
                if results_dict['action'] == 'click':
                    if results_dict['selector_count'] == 1:
                        self.wait_for_click(results_dict['selectors'][0])
                    if results_dict['selector_count'] == 2:
                        try:
                            self.wait_for_click(results_dict['selectors'][0])
                        except Exception as e:
                            self.wait_for_click(results_dict['selectors'][1])
        except Exception as e:
            self.err.err_full_traceback(e, 'Unable to add item: ' + item_dict['prodNumber'] + ' in prod chooser')
            self.log('info', '[INFO] [ERROR] Unable to add item: ' + item_dict['prodNumber'] + ' in prod chooser. Traceback: ' + str(e))


    def stProdChooserFinish(self):
        try:
            self.wait_for_click(stprodchooser.prodchooserFinishButton)
        except Exception as e:
            self.err.err_basic('Unable to click finish, please click finish and click ok to continue', 'Error')
            self.log('info', '[INFO] [ERROR] Unable to click finish'), self.log('warning', str(e))
            self.current_error_count += 1

    def getResults(self, _selector):
        self.wait_for(_selector)
        data = self.css(_selector).get_attribute('innerHTML')
        if data == 'Loading...':
            for _i in range(6):
                if _i < 5:
                    time.sleep(3)
                    data = self.css(_selector).get_attribute('innerHTML')
                    if data == 'Loading...':
                        continue
                    else:
                        if data != "No records":
                            data = data.split(' ')[0]
                            return [int(data.split('-')[0]), int(data.split('-')[1])]  # [min val, max val]
                        else:
                            return data
                elif _i >= 5:
                    self.err.err_custom('Could not get number of records returned.', 'Error', 'MB_OK', 'ICON_EXCLAIM')
                    break
        elif data != "No records":
            data = data.split(' ')[0]
            return [int(data.split('-')[0]), int(data.split('-')[1])] #[min val, max val]
        elif data == "No records":
            self.err.err_custom('No Records found for this item.', 'Error', 'MB_OK', 'ICON_EXCLAIM')
            self.log('info', '[INFO] No Records found for this item.')
            return data
        else:
            data = data.split(' ')[0]
            return [int(data.split('-')[0]), int(data.split('-')[1])] #[min val, max val]

    def go_back(self):
        self.driver.execute_script("window.history.go(-1)")

    def input_action_chain(self, delay=0.4, init_action=None, input_selector='None', input_text='None'):

        action_chain = init_action #Clears action chain
        time.sleep(delay)
        action_input_alias = self.css(input_selector)
        action_input_alias.click()
        action_chain = ActionChains(self.driver)
        action_chain.move_to_element(action_input_alias).send_keys(input_text).perform()
        action_chain = init_action #Clears action chain


    def stMutation(self, dict, data):
        #Click Mutate
        self.wait_for_click(stmutate.stMutateButton)

        #Adjust Quantity
        self.actions = None
        self.quantity = self.css(stmutate.stMutateQuantity)
        self.quantity.click()
        self.actions = ActionChains(self.driver)
        self.clickinto = self.actions.move_to_element(self.quantity).send_keys('1').perform()
        self.actions = None

        #Add Note
        self.actions = None
        self.note = self.css(stmutate.stMutateNote)
        self.note.click()
        self.actions = ActionChains(self.driver)
        self.clickinto = self.actions.move_to_element(self.note).send_keys(stmutate.stnote).perform()
        self.actions = None

        #click save, finish mutation
        try:
            self.wait_for_click(stmutate.stMutateSavebutton)
            time.sleep(2)
            print('Product: ' + dict['prodnumber'] + ' Lot: ' + dict['lot_serial'] + ' Case: ' + str(data[0]).rstrip('.0') + ' Successfully Mutated' + '\n')
            self.log('info', '[INFO] Product: ' + str(dict['prodnumber']) + ' Lot: ' + str(dict['lot_serial']) + ' Successfully Mutated')
            self.driver.execute_script("window.history.go(-1)")
            try:
                self.wait_for_click(stmutate.stMutateCancelButton)
            except:
                None #No cancel button, move on
            self.stStockTabButton()
        except Exception as e:
            self.err.err_basic('Unable to mutate Product: ' + dict['prodnumber'] + ' Lot: ' + dict['lot_serial'] + ' Case: ' + str(data[0]).rstrip('.0') + ', please mutate and hit enter to continue', 'Mutation Error')
            self.log('warning', '[CRITICAL ERROR] Unable to mutate Product: ' + str(dict['prodnumber']) + ' Lot: ' + str(dict['lot_serial']) + ' , please mutate and hit enter to continue')
            self.log('warning', str(e))


    def stTableCheckBoxes(self, N):
        self.wait_for_click(self.web_logic.stock_tab_table_checkboxes(N))

    def parse_stock_table(self):
        dr = self.getResults(st.stStockTabResults)
        results = {'total_rows': dr[1], "items": []}
        if 'No records' in dr:
            return "No Records Found"
        elif dr == None:
            raise NoRecordsFoundError('Stock Table range <' + str(type(dr)) + '>')
        else:
            pass
        for y in range(dr[0], dr[1] + 1):
            print(Fore.LIGHTYELLOW_EX + 'Parsing row: ' + str(y) + ' of ' + str(dr[1]))

            tableProdNumber = {"product": self.css(sttable.stTableProductFront + str(y) + sttable.stTableProductBack).get_attribute('innerHTML'), "path": sttable.stTableProductFront + str(y) + sttable.stTableProductBack}
            tableDescription = {"description": self.css(sttable.stTableDescriptionFront + str(y) + sttable.stTableDescriptionBack).get_attribute('innerHTML'), "path": sttable.stTableDescriptionFront + str(y) + sttable.stTableDescriptionBack}
            tableLotSerial = {"lot_serial": self.css(sttable.stTableLotSerialFront + str(y) + sttable.stTableLotSerialBack).get_attribute('innerHTML'), "path": sttable.stTableLotSerialFront + str(y) + sttable.stTableLotSerialBack}
            tableContainer = {"container": self.css(sttable.stTableContainerFront + str(y) + sttable.stTableContainerBackInfo).get_attribute('innerHTML'), "path": sttable.stTableContainerFront + str(y) + sttable.stTableContainerBackInfo}
            tableContainerType = {"container_type": self.css(sttable.stTableContainerFront + str(y) + sttable.stTableContainerBackType).get_attribute('innerHTML'),"path": sttable.stTableContainerFront + str(y) + sttable.stTableContainerBackType}
            tableLocation = {"location": self.css(sttable.stTableLocationFront + str(y) + sttable.stTableLocationBackInfo).get_attribute('innerHTML'), "path": sttable.stTableLocationFront + str(y) + sttable.stTableLocationBackInfo}
            tableLocationType = {"location_type": self.css(sttable.stTableLocationFront + str(y) + sttable.stTableLocationBackLocType).get_attribute('innerHTML'), "path": sttable.stTableLocationFront + str(y) + sttable.stTableLocationBackLocType}

            results["items"].append({"row": y ,"data": {"product_info": tableProdNumber,
                                                        "description_info": tableDescription,
                                                        "lot_serial_info": tableLotSerial,
                                                        "container_info": tableContainer,
                                                        "container_type_info": tableContainerType,
                                                        "location_info": tableLocation,
                                                        "location_type_info": tableLocationType}})

        return results

    def stTableParsing(self, dict, funcType, bins="No"):
        if bins == "No":
            data = [dict['caseNumber'], dict['lotNumber'], dict['serialNumber'], "none"]
        elif bins == "Yes":
            data = [dict['caseNumber'], dict['lotNumber'], dict['serialNumber'], dict['binlocation']]
        dr = self.getResults(st.stStockTabResults)
        if 'No records' in dr:
            raise NoRecordsFoundError('Stock Table range <' + str(dr[0]) + ' , ' + str(dr[1] + 1) + '>')
        elif dr == None:
            raise NoRecordsFoundError('Stock Table range <' + str(type(dr)) + '>')
        else:
            pass
        for y in range(dr[0], dr[1] + 1):
            print(Fore.LIGHTYELLOW_EX + 'Parsing row: ' + str(y) + ' of ' + str(dr[1]))
            tableProdNumber = self.css(sttable.stTableProductFront + str(y) + sttable.stTableProductBack).get_attribute('innerHTML')
            tableDescription = self.css(sttable.stTableDescriptionFront + str(y) + sttable.stTableDescriptionBack).get_attribute('innerHTML')
            tableLotSerial = self.css(sttable.stTableLotSerialFront + str(y) + sttable.stTableLotSerialBack).get_attribute('innerHTML')
            tableContainer = self.css(sttable.stTableContainerFront + str(y) + sttable.stTableContainerBackInfo).get_attribute('innerHTML')
            tableLocation = self.css(sttable.stTableLocationFront + str(y) + sttable.stTableLocationBackInfo).get_attribute('innerHTML')
            tableLocationType = self.css(sttable.stTableLocationFront + str(y) + sttable.stTableLocationBackLocType).get_attribute('innerHTML')

            if funcType == 'mutation':
                _result = self.stBoolDataMutation({'rowNumber': str(y), 'prodnumber': tableProdNumber, 'description': tableDescription, 'lot_serial': tableLotSerial,
                                                   'container': tableContainer, 'location': tableLocation, 'location_type': tableLocationType}, data)
                if _result == True:
                    return True
                    break
                elif _result == False:
                    if y >= dr[1] + 1:
                        return False

    def stBoolDataMutation(self, dict, data):
        _return = None
        results = self.web_logic.bool_data_from_table(dict, data)
        if results == None:
            _return = False
            pass
        if results['match_found'] == True:
            print(Fore.LIGHTGREEN_EX + 'Found Match!' + '\n')
            lv.formatListToTable(dict, lv.formatDictotList(dict), Fore.LIGHTYELLOW_EX)
            self.wait_for_click(results['data']['checkbox_selector'])
            try:
                self.stMutation(dict, data)
                _return = True
                return _return
            except Exception as e:
                self.current_error_count += 1
                self.err.err_traceback(e, 'Unable to mutate Product: ' + dict['prodnumber'] + ' Lot: ' + dict['lot_serial'] + ' Case: ' + str(data[1]).rstrip('.0') + ', please mutate and hit enter to continue')
                self.log('warning', '[CRITICAL ERROR] Unable to mutate Product: ' + str(dict['prodnumber']) + ' Lot: ' + str(dict['lot_serial']))
                _return = False
        if results['match_found'] == False:
            _return = False
        return _return

    def wait_for_click(self, selector):
        i = 0
        while i < 10:
            try:
                self.css(selector).click()
                i = 11
                break
            except:
                if i >= 10:
                    self.log('warning', "Selector timed out : {" + selector + "} ")
                    raise WaitTimeOutError(str(10), selector)
                    break
                else:
                    i += 1
                    time.sleep(0.6)
                    continue

    def wait_for(self, selector):
        i = 0
        while i < 10:
            try:
                self.css(selector)
                i = 11
                break
            except:
                if i >= 10:
                    self.log('warning', "Selector timed out : {" + selector + "} ")
                    raise WaitTimeOutError(str(10), selector)
                    break
                else:
                    i += 1
                    time.sleep(0.3)
                    continue

    def mutate(self, _dict):

        product_number = _dict['productNumber']
        self.stProdChooserSelect()

        self.stProdChooserSearch(product_number)

        self.stProdChooserAddItem({'prodNumber': _dict['productNumber'], 'description': _dict['description']})

        self.stProdChooserFinish()
        self.stSearchButton()

        try:
            _results = self.stTableParsing(_dict, 'mutation', bins=_dict['bin?'])
            if _results != True:
                self.err.err_custom('Item Product: ' + _dict['productNumber'] + ', was never mutated. Please resolve the issue manually.', 'Alert!', 'MB_OK', 'ICON_EXCLAIM')
                self.log('warning', 'Item Product: ' + _dict['productNumber'] + ', was never mutated. Please resolve the issue manually.')
        except Exception as e:
            print(Fore.LIGHTRED_EX + 'Failed to parse table. Skipping item [{}]'.format(_dict['productNumber']))
            print(Fore.LIGHTMAGENTA_EX + str(e))



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 15 16:16:58 2016

@author: doarni
"""

import ctypes as c


def msgBox(msg, title, button, icon):
    dictButtons = {'MB_OK': 0x0,
                    'MB_OKCXL': 0x01,
                    'MB_YESNOCXL': 0x03,
                    'MB_YESNO': 0x04,
                    'MB_HELP': 0x4000}
                    
    dictIcons = {'ICON_EXCLAIM': 0x30,
                 'ICON_INFO': 0x40,
                 'ICON_STOP': 0x10}
    
    dictIDS = {'IDOK': 0,
               'IDCANCEL':  2,
               'IDABORT': 3,
               'IDYES': 6,
               'IDNO': 7}
            
    c.windll.user32.MessageBoxW(0, title, msg, dictButtons[button] | dictIcons[icon])




stTableContainerInit = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(1) > td:nth-child(5) > div > table > tbody > tr > td.GLNOMTPDG5D'
stTableCheckBoxRowOddFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableLotSerialFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableCheckBoxRowOddBack = ') > td.GLNOMTPDHM.GLNOMTPDEO > div > span > img'
stTableZSMSHoldBackOdd = ') > td.GLNOMTPDHM GLNOMTPDNO > div'
stTableProductFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableLocationBackLocType = ') > td:nth-child(6) > div > table > tbody > tr > td.GLNOMTPDF5D'
stTableZSMSHoldInit = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(1) > td.GLNOMTPDHM.GLNOMTPDBP.GLNOMTPDNO > div'
stTableDescriptionBack = ') > td:nth-child(3) > div'
stTableCheckBoxRowEvenFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableQAVFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableLocationBackInfo = ') > td:nth-child(6) > div > table > tbody > tr > td.GLNOMTPDG5D > span'
stTableQRFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableDescriptionFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableQAVBack = ') > td:nth-child(8) > div'
stTableCheckBoxInit = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child(1) > td.GLNOMTPDHM.GLNOMTPDEO.GLNOMTPDBP > div > span > img'
stTableQOHFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableLocationFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableZSMSHoldBackEven = ') > td.GLNOMTPDGM GLNOMTPDNO > div'
stTableLotSerialBack = ') > td:nth-child(4) > div'
stTableZSMSHoldFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableQRBack = ') > td:nth-child(9) > div'
stTableQOHBack = ') > td:nth-child(7) > div'
stTableContainerFront = 'body > div:nth-child(8) > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(6) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(3) > div > div:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(2) > div > div:nth-child(2) > div > div:nth-child(3) > div > div > table > tbody > tr:nth-child('
stTableCheckBoxRowEvenBack = ') > td.GLNOMTPDGM.GLNOMTPDEO > div > span > img'
stTableProductBack = ') > td:nth-child(2) > span'
stTableContainerBackType = ') > td:nth-child(5) > div > table > tbody > tr > td.GLNOMTPDF5D'
stTableContainerBackInfo = ') > td:nth-child(5) > div > table > tbody > tr > td.GLNOMTPDG5D > span'



# -*- coding: utf-8 -*-
"""
Created on Wed Dec  7 12:30:48 2016

@author: doarni
"""


import pySMS.excel.ui.openfile as op
import os

class openFile():
    
    def __init__(self):
        self.path = os.path.dirname(os.path.realpath(__file__))
        self.fname = None
        
    def openFileDialog(self):
        self.fname = op.dialog()
        return self.fname



import json
import gzip
import os


class Export:

    def __init__(self):
        self.data_files_list = self.get_data_files_list()

    @staticmethod
    def get_data_files_list():
        data = {}
        files = [f for f in os.listdir(os.path.dirname(os.path.realpath(__file__)))]
        for file in files:
            data[file] = file
        return data

    @staticmethod
    def unzip_from_gzip(file):
        with gzip.open(os.path.dirname(os.path.realpath(__file__)) + '\\' + file, 'rb') as f:
            file_content = f.read()
        return {'file': file, 'content': file_content, 'type': type(file_content)}

    @staticmethod
    def zip_to_gzip(file, data_obj):
        _f = file.split('.')
        with gzip.open(os.path.dirname(os.path.realpath(__file__)) + '\\' + file + ".gz", 'wb')as f:
            f.write(data_obj)
        f.close()

    @staticmethod
    def import_json_file(file):
        with open(file)as data_file:
            content = json.load(data_file)
        return {'file': file, 'content': content, 'type': type(content)}

    @staticmethod
    def convert_to_dict(obj):
        return json.loads(obj)

    @staticmethod
    def convert_to_gzip_obj(obj):
        gzip_obj = json.dumps(obj)
        return {'origin_type': type(obj), 'new_type': type(gzip_obj), 'obj': gzip_obj}



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 15 15:16:15 2016

@author: doarni
"""

import pyHook
from pyHook import HookManager, GetKeyState, HookConstants
import pySMS.keys as keys
import pythoncom
import sys

def OnKeyboardEvent(event):
    # in case you want to debug: uncomment next line
    # print repr(event), event.KeyID, HookConstants.IDToName(event.KeyID), event.ScanCode , event.Ascii, event.flags
    if GetKeyState(HookConstants.VKeyToID('VK_LSHIFT')) and event.KeyID == HookConstants.VKeyToID('VK_SNAPSHOT'):
        print("shift + snapshot pressed")
    elif GetKeyState(HookConstants.VKeyToID('VK_CONTROL')) and HookConstants.IDToName(event.KeyID) == 'D':
        print("ctrl + d pressed")
        sys.exit()
    elif GetKeyState(HookConstants.VKeyToID(keys.VK_LALT)) and HookConstants.IDToName(event.KeyID) == 'Q':
        print("alt + q pressed")
        
    return True


if __name__ == '__main__':
    hm = pyHook.HookManager()
    hm.KeyDown = OnKeyboardEvent
    hm.HookKeyboard()
    pythoncom.PumpMessages()
else:
    print("This module cannot be imported")
    print('This it to be used to test pyHook')
    raise ImportError
